@@ -18,10 +18,15 @@
 #[macro_use]
 extern crate napi_derive;
 
+mod capability;
+
 use std::collections::HashMap;
 use std::str::FromStr;
 use std::time::Duration;
 
+use opendal::raw::oio::BlockingRead;
+use opendal::raw::oio::ReadExt;
+
 use futures::TryStreamExt;
 use napi::bindgen_prelude::*;
 
@@ -41,9 +46,25 @@ impl Operator {
             .map_err(format_napi_error)?;
         let options = options.unwrap_or_default();
 
-        opendal::Operator::via_map(scheme, options)
-            .map_err(format_napi_error)
-            .map(Operator)
+        let mut op = opendal::Operator::via_map(scheme, options).map_err(format_napi_error)?;
+
+        if !op.info().full_capability().blocking {
+            if let Ok(handle) = tokio::runtime::Handle::try_current() {
+                let _guard = handle.enter();
+                op = op.layer(
+                    opendal::layers::BlockingLayer::create()
+                        .expect("blocking layer must be created"),
+                );
+            }
+        }
+
+        Ok(Operator(op))
+    }
+
+    /// Get current operator(service)'s full capability.
+    #[napi]
+    pub fn capability(&self) -> Result<capability::Capability> {
+        Ok(capability::Capability::new(self.0.info().full_capability()))
     }
 
     /// Get current path's metadata **without cache** directly.
@@ -158,6 +179,15 @@ impl Operator {
         Ok(res.into())
     }
 
+    /// Create a reader to read the given path.
+    ///
+    /// It could be used to read large file in a streaming way.
+    #[napi]
+    pub async fn reader(&self, path: String) -> Result<Reader> {
+        let r = self.0.reader(&path).await.map_err(format_napi_error)?;
+        Ok(Reader(r))
+    }
+
     /// Read the whole path into a buffer synchronously.
     ///
     /// ### Example
@@ -170,6 +200,15 @@ impl Operator {
         Ok(res.into())
     }
 
+    /// Create a reader to read the given path synchronously.
+    ///
+    /// It could be used to read large file in a streaming way.
+    #[napi]
+    pub fn reader_sync(&self, path: String) -> Result<BlockingReader> {
+        let r = self.0.blocking().reader(&path).map_err(format_napi_error)?;
+        Ok(BlockingReader(r))
+    }
+
     /// Write bytes into path.
     ///
     /// ### Example
@@ -187,6 +226,24 @@ impl Operator {
         self.0.write(&path, c).await.map_err(format_napi_error)
     }
 
+    /// Write multiple bytes into path.
+    ///
+    /// It could be used to write large file in a streaming way.
+    #[napi]
+    pub async fn writer(&self, path: String) -> Result<Writer> {
+        let w = self.0.writer(&path).await.map_err(format_napi_error)?;
+        Ok(Writer(w))
+    }
+
+    /// Write multiple bytes into path synchronously.
+    ///
+    /// It could be used to write large file in a streaming way.
+    #[napi]
+    pub fn writer_sync(&self, path: String) -> Result<BlockingWriter> {
+        let w = self.0.blocking().writer(&path).map_err(format_napi_error)?;
+        Ok(BlockingWriter(w))
+    }
+
     /// Write bytes into path synchronously.
     ///
     /// ### Example
@@ -311,7 +368,7 @@ impl Operator {
         Ok(Lister(
             self.0
                 .lister_with(&path)
-                .delimiter("")
+                .recursive(true)
                 .await
                 .map_err(format_napi_error)?,
         ))
@@ -343,7 +400,7 @@ impl Operator {
             self.0
                 .blocking()
                 .lister_with(&path)
-                .delimiter("")
+                .recursive(true)
                 .call()
                 .map_err(format_napi_error)?,
         ))
@@ -455,7 +512,6 @@ impl Operator {
             self.0
                 .blocking()
                 .lister_with(&path)
-                .delimiter("")
                 .call()
                 .map_err(format_napi_error)?,
         ))
@@ -531,6 +587,7 @@ impl Operator {
     }
 }
 
+/// Entry returned by Lister or BlockingLister to represent a path and it's relative metadata.
 #[napi]
 pub struct Entry(opendal::Entry);
 
@@ -543,6 +600,7 @@ impl Entry {
     }
 }
 
+/// Metadata carries all metadata associated with a path.
 #[napi]
 pub struct Metadata(opendal::Metadata);
 
@@ -599,6 +657,132 @@ impl Metadata {
     }
 }
 
+/// BlockingReader is designed to read data from given path in an blocking
+/// manner.
+#[napi]
+pub struct BlockingReader(opendal::BlockingReader);
+
+#[napi]
+impl BlockingReader {
+    #[napi]
+    pub fn read(&mut self, mut buf: Buffer) -> Result<usize> {
+        self.0.read(buf.as_mut()).map_err(format_napi_error)
+    }
+}
+
+/// Reader is designed to read data from given path in an asynchronous
+/// manner.
+#[napi]
+pub struct Reader(opendal::Reader);
+
+#[napi]
+impl Reader {
+    /// # Safety
+    ///
+    /// > &mut self in async napi methods should be marked as unsafe
+    ///
+    /// Read bytes from this reader into given buffer.
+    #[napi]
+    pub async unsafe fn read(&mut self, mut buf: Buffer) -> Result<usize> {
+        self.0.read(buf.as_mut()).await.map_err(format_napi_error)
+    }
+}
+
+/// BlockingWriter is designed to write data into given path in an blocking
+/// manner.
+#[napi]
+pub struct BlockingWriter(opendal::BlockingWriter);
+
+#[napi]
+impl BlockingWriter {
+    /// # Safety
+    ///
+    /// > &mut self in async napi methods should be marked as unsafe
+    ///
+    /// Write bytes into this writer.
+    ///
+    /// ### Example
+    /// ```javascript
+    /// const writer = await op.writer("path/to/file");
+    /// await writer.write(Buffer.from("hello world"));
+    /// await writer.close();
+    /// ```
+    #[napi]
+    pub unsafe fn write(&mut self, content: Either<Buffer, String>) -> Result<()> {
+        let c = match content {
+            Either::A(buf) => buf.as_ref().to_owned(),
+            Either::B(s) => s.into_bytes(),
+        };
+        self.0.write(c).map_err(format_napi_error)
+    }
+
+    /// # Safety
+    ///
+    /// > &mut self in async napi methods should be marked as unsafe
+    ///
+    /// Close this writer.
+    ///
+    /// ### Example
+    ///
+    /// ```javascript
+    /// const writer = op.writerSync("path/to/file");
+    /// writer.write(Buffer.from("hello world"));
+    /// writer.close();
+    /// ```
+    #[napi]
+    pub unsafe fn close(&mut self) -> Result<()> {
+        self.0.close().map_err(format_napi_error)
+    }
+}
+
+/// Writer is designed to write data into given path in an asynchronous
+/// manner.
+#[napi]
+pub struct Writer(opendal::Writer);
+
+#[napi]
+impl Writer {
+    /// # Safety
+    ///
+    /// > &mut self in async napi methods should be marked as unsafe
+    ///
+    /// Write bytes into this writer.
+    ///
+    /// ### Example
+    /// ```javascript
+    /// const writer = await op.writer("path/to/file");
+    /// await writer.write(Buffer.from("hello world"));
+    /// await writer.close();
+    /// ```
+    #[napi]
+    pub async unsafe fn write(&mut self, content: Either<Buffer, String>) -> Result<()> {
+        let c = match content {
+            Either::A(buf) => buf.as_ref().to_owned(),
+            Either::B(s) => s.into_bytes(),
+        };
+        self.0.write(c).await.map_err(format_napi_error)
+    }
+
+    /// # Safety
+    ///
+    /// > &mut self in async napi methods should be marked as unsafe
+    ///
+    /// Close this writer.
+    ///
+    /// ### Example
+    /// ```javascript
+    /// const writer = await op.writer("path/to/file");
+    /// await writer.write(Buffer.from("hello world"));
+    /// await writer.close();
+    /// ```
+    #[napi]
+    pub async unsafe fn close(&mut self) -> Result<()> {
+        self.0.close().await.map_err(format_napi_error)
+    }
+}
+
+/// Lister is designed to list entries at given path in an asynchronous
+/// manner.
 #[napi]
 pub struct Lister(opendal::Lister);
 
@@ -621,6 +805,8 @@ impl Lister {
     }
 }
 
+/// BlockingLister is designed to list entries at given path in a blocking
+/// manner.
 #[napi]
 pub struct BlockingLister(opendal::BlockingLister);
 
@@ -641,6 +827,7 @@ impl BlockingLister {
     }
 }
 
+/// PresignedRequest is a presigned request return by `presign`.
 #[napi(object)]
 pub struct PresignedRequest {
     /// HTTP method of this request.
@@ -675,6 +862,144 @@ impl PresignedRequest {
     }
 }
 
+pub trait NodeLayer: Send + Sync {
+    fn layer(&self, op: opendal::Operator) -> opendal::Operator;
+}
+
+/// A public layer wrapper
+#[napi]
+pub struct Layer {
+    inner: Box<dyn NodeLayer>,
+}
+
+#[napi]
+impl Operator {
+    /// Add a layer to this operator.
+    #[napi]
+    pub fn layer(&self, layer: External<Layer>) -> Result<Self> {
+        Ok(Self(layer.inner.layer(self.0.clone())))
+    }
+}
+
+impl NodeLayer for opendal::layers::RetryLayer {
+    fn layer(&self, op: opendal::Operator) -> opendal::Operator {
+        op.layer(self.clone())
+    }
+}
+
+/// Retry layer
+///
+/// Add retry for temporary failed operations.
+///
+/// # Notes
+///
+/// This layer will retry failed operations when [`Error::is_temporary`]
+/// returns true. If operation still failed, this layer will set error to
+/// `Persistent` which means error has been retried.
+///
+/// `write` and `blocking_write` don't support retry so far, visit [this issue](https://github.com/apache/incubator-opendal/issues/1223) for more details.
+///
+/// # Examples
+///
+/// ```javascript
+/// const op = new Operator("file", { root: "/tmp" })
+///
+/// const retry = new RetryLayer();
+/// retry.max_times = 3;
+/// retry.jitter = true;
+///
+/// op.layer(retry.build());
+/// ```
+#[derive(Default)]
+#[napi]
+pub struct RetryLayer {
+    jitter: bool,
+    max_times: Option<u32>,
+    factor: Option<f64>,
+    max_delay: Option<f64>,
+    min_delay: Option<f64>,
+}
+
+#[napi]
+impl RetryLayer {
+    #[napi(constructor)]
+    pub fn new() -> Self {
+        Self::default()
+    }
+
+    /// Set jitter of current backoff.
+    ///
+    /// If jitter is enabled, ExponentialBackoff will add a random jitter in `[0, min_delay)
+    /// to current delay.
+    #[napi(setter)]
+    pub fn jitter(&mut self, v: bool) {
+        self.jitter = v;
+    }
+
+    /// Set max_times of current backoff.
+    ///
+    /// Backoff will return `None` if max times is reaching.
+    #[napi(setter)]
+    pub fn max_times(&mut self, v: u32) {
+        self.max_times = Some(v);
+    }
+
+    /// Set factor of current backoff.
+    ///
+    /// # Panics
+    ///
+    /// This function will panic if input factor smaller than `1.0`.
+    #[napi(setter)]
+    pub fn factor(&mut self, v: f64) {
+        self.factor = Some(v);
+    }
+
+    /// Set max_delay of current backoff.
+    ///
+    /// Delay will not increasing if current delay is larger than max_delay.
+    ///
+    /// # Notes
+    ///
+    /// - The unit of max_delay is millisecond.
+    #[napi(setter)]
+    pub fn max_delay(&mut self, v: f64) {
+        self.max_delay = Some(v);
+    }
+
+    /// Set min_delay of current backoff.
+    ///
+    /// # Notes
+    ///
+    /// - The unit of min_delay is millisecond.
+    #[napi(setter)]
+    pub fn min_delay(&mut self, v: f64) {
+        self.min_delay = Some(v);
+    }
+
+    #[napi]
+    pub fn build(&self) -> External<Layer> {
+        let mut l = opendal::layers::RetryLayer::default();
+        if self.jitter {
+            l = l.with_jitter();
+        }
+        if let Some(max_times) = self.max_times {
+            l = l.with_max_times(max_times as usize);
+        }
+        if let Some(factor) = self.factor {
+            l = l.with_factor(factor as f32);
+        }
+        if let Some(max_delay) = self.max_delay {
+            l = l.with_max_delay(Duration::from_millis(max_delay as u64));
+        }
+        if let Some(min_delay) = self.min_delay {
+            l = l.with_min_delay(Duration::from_millis(min_delay as u64));
+        }
+
+        External::new(Layer { inner: Box::new(l) })
+    }
+}
+
+/// Format opendal error to napi error.
 fn format_napi_error(err: opendal::Error) -> Error {
     Error::from_reason(format!("{}", err))
 }
