@@ -166,3 +166,15 @@ pub mod rfc_3197_config {}
 
 #[doc = include_str!("3232_align_list_api.md")]
 pub mod rfc_3232_align_list_api {}
+
+#[doc = include_str!("3243_list_prefix.md")]
+pub mod rfc_3243_list_prefix {}
+
+#[doc = include_str!("3356_lazy_reader.md")]
+pub mod rfc_3356_lazy_reader {}
+
+#[doc = include_str!("3526_list_recursive.md")]
+pub mod rfc_3526_list_recursive {}
+
+#[doc = include_str!("3574_concurrent_stat_in_list.md")]
+pub mod rfc_3574_concurrent_stat_in_list {}
