@@ -27,12 +27,11 @@ use std::task::Poll;
 use async_trait::async_trait;
 use bytes::Bytes;
 
-use crate::raw::oio::into_flat_page;
-use crate::raw::oio::into_hierarchy_page;
-use crate::raw::oio::ByRangeSeekableReader;
-use crate::raw::oio::Entry;
-use crate::raw::oio::FlatPager;
-use crate::raw::oio::HierarchyPager;
+use crate::raw::oio::FileReader;
+use crate::raw::oio::FlatLister;
+use crate::raw::oio::HierarchyLister;
+use crate::raw::oio::LazyReader;
+use crate::raw::oio::RangeReader;
 use crate::raw::oio::StreamableReader;
 use crate::raw::*;
 use crate::*;
@@ -50,6 +49,10 @@ use crate::*;
 ///
 /// So far `CompleteLayer` will do the following things:
 ///
+/// ## Stat Completion
+///
+/// Not all services support stat dir natively, but we can simulate it via list.
+///
 /// ## Read Completion
 ///
 /// OpenDAL requires all reader implements [`oio::Read`] and
@@ -101,11 +104,11 @@ use crate::*;
 /// them. CompleteLayer will add those capabilities in a zero cost way.
 ///
 /// Underlying services will return [`Capability`] to indicate the
-/// features that returning pagers support.
+/// features that returning listers support.
 ///
-/// - If both `list_with_delimiter_slash` and `list_without_delimiter`, return directly.
-/// - If only `list_without_delimiter`, with [`oio::to_flat_pager`].
-/// - if only `list_with_delimiter_slash`, with [`oio::to_hierarchy_pager`].
+/// - If both `list_without_recursive` and `list_with_recursive`, return directly.
+/// - If only `list_with_recursive`, with [`oio::to_flat_lister`].
+/// - if only `list_without_recursive`, with [`oio::to_hierarchy_lister`].
 /// - If neither not supported, something must be wrong for `list` is true.
 ///
 /// ## Capability Check
@@ -116,10 +119,10 @@ use crate::*;
 pub struct CompleteLayer;
 
 impl<A: Accessor> Layer<A> for CompleteLayer {
-    type LayeredAccessor = CompleteReaderAccessor<A>;
+    type LayeredAccessor = CompleteAccessor<A>;
 
     fn layer(&self, inner: A) -> Self::LayeredAccessor {
-        CompleteReaderAccessor {
+        CompleteAccessor {
             meta: inner.info(),
             inner: Arc::new(inner),
         }
@@ -127,18 +130,18 @@ impl<A: Accessor> Layer<A> for CompleteLayer {
 }
 
 /// Provide complete wrapper for backend.
-pub struct CompleteReaderAccessor<A: Accessor> {
+pub struct CompleteAccessor<A: Accessor> {
     meta: AccessorInfo,
     inner: Arc<A>,
 }
 
-impl<A: Accessor> Debug for CompleteReaderAccessor<A> {
+impl<A: Accessor> Debug for CompleteAccessor<A> {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         self.inner.fmt(f)
     }
 }
 
-impl<A: Accessor> CompleteReaderAccessor<A> {
+impl<A: Accessor> CompleteAccessor<A> {
     fn new_unsupported_error(&self, op: impl Into<&'static str>) -> Error {
         let scheme = self.meta.scheme();
         let op = op.into();
@@ -149,7 +152,138 @@ impl<A: Accessor> CompleteReaderAccessor<A> {
         .with_operation(op)
     }
 
-    async fn complete_reader(
+    async fn complete_create_dir(&self, path: &str, args: OpCreateDir) -> Result<RpCreateDir> {
+        let capability = self.meta.full_capability();
+        if capability.create_dir {
+            return self.inner().create_dir(path, args).await;
+        }
+        if capability.write_can_empty && capability.list {
+            let (_, mut w) = self.inner.write(path, OpWrite::default()).await?;
+            oio::WriteExt::close(&mut w).await?;
+            return Ok(RpCreateDir::default());
+        }
+
+        Err(self.new_unsupported_error(Operation::CreateDir))
+    }
+
+    fn complete_blocking_create_dir(&self, path: &str, args: OpCreateDir) -> Result<RpCreateDir> {
+        let capability = self.meta.full_capability();
+        if capability.create_dir && capability.blocking {
+            return self.inner().blocking_create_dir(path, args);
+        }
+        if capability.write_can_empty && capability.list && capability.blocking {
+            let (_, mut w) = self.inner.blocking_write(path, OpWrite::default())?;
+            oio::BlockingWrite::close(&mut w)?;
+            return Ok(RpCreateDir::default());
+        }
+
+        Err(self.new_unsupported_error(Operation::BlockingCreateDir))
+    }
+
+    async fn complete_stat(&self, path: &str, args: OpStat) -> Result<RpStat> {
+        let capability = self.meta.full_capability();
+        if !capability.stat {
+            return Err(self.new_unsupported_error(Operation::Stat));
+        }
+
+        if path == "/" {
+            return Ok(RpStat::new(Metadata::new(EntryMode::DIR)));
+        }
+
+        // Forward to inner if create_dir is supported.
+        if path.ends_with('/') && capability.create_dir {
+            let meta = self.inner.stat(path, args).await?.into_metadata();
+
+            if meta.is_file() {
+                return Err(Error::new(
+                    ErrorKind::NotFound,
+                    "stat expected a directory, but found a file",
+                ));
+            }
+
+            return Ok(RpStat::new(Metadata::new(EntryMode::DIR)));
+        }
+
+        // Otherwise, we can simulate stat dir via `list`.
+        if path.ends_with('/') && capability.list_with_recursive {
+            let (_, mut l) = self
+                .inner
+                .list(
+                    path.trim_end_matches('/'),
+                    OpList::default().with_recursive(true).with_limit(1),
+                )
+                .await?;
+
+            return if oio::ListExt::next(&mut l).await?.is_some() {
+                Ok(RpStat::new(Metadata::new(EntryMode::DIR)))
+            } else {
+                Err(Error::new(
+                    ErrorKind::NotFound,
+                    "the directory is not found",
+                ))
+            };
+        }
+
+        // Forward to underlying storage directly since we don't know how to handle stat dir.
+        self.inner.stat(path, args).await.map(|v| {
+            v.map_metadata(|m| {
+                let bit = m.metakey();
+                m.with_metakey(bit | Metakey::Complete)
+            })
+        })
+    }
+
+    fn complete_blocking_stat(&self, path: &str, args: OpStat) -> Result<RpStat> {
+        let capability = self.meta.full_capability();
+        if !capability.stat {
+            return Err(self.new_unsupported_error(Operation::Stat));
+        }
+
+        if path == "/" {
+            return Ok(RpStat::new(Metadata::new(EntryMode::DIR)));
+        }
+
+        // Forward to inner if create dir is supported.
+        if path.ends_with('/') && capability.create_dir {
+            let meta = self.inner.blocking_stat(path, args)?.into_metadata();
+
+            if meta.is_file() {
+                return Err(Error::new(
+                    ErrorKind::NotFound,
+                    "stat expected a directory, but found a file",
+                ));
+            }
+
+            return Ok(RpStat::new(Metadata::new(EntryMode::DIR)));
+        }
+
+        // Otherwise, we can simulate stat a dir path via `list`.
+        if path.ends_with('/') && capability.list_with_recursive {
+            let (_, mut l) = self.inner.blocking_list(
+                path.trim_end_matches('/'),
+                OpList::default().with_recursive(true).with_limit(1),
+            )?;
+
+            return if oio::BlockingList::next(&mut l)?.is_some() {
+                Ok(RpStat::new(Metadata::new(EntryMode::DIR)))
+            } else {
+                Err(Error::new(
+                    ErrorKind::NotFound,
+                    "the directory is not found",
+                ))
+            };
+        }
+
+        // Forward to underlying storage directly since we don't know how to handle stat dir.
+        self.inner.blocking_stat(path, args).map(|v| {
+            v.map_metadata(|m| {
+                let bit = m.metakey();
+                m.with_metakey(bit | Metakey::Complete)
+            })
+        })
+    }
+
+    async fn complete_read(
         &self,
         path: &str,
         args: OpRead,
@@ -162,47 +296,30 @@ impl<A: Accessor> CompleteReaderAccessor<A> {
         let seekable = capability.read_can_seek;
         let streamable = capability.read_can_next;
 
-        let range = args.range();
-        let (rp, r) = self.inner.read(path, args).await?;
-        let content_length = rp.metadata().content_length();
-
         match (seekable, streamable) {
-            (true, true) => Ok((rp, CompleteReader::AlreadyComplete(r))),
+            (true, true) => {
+                let r = LazyReader::new(self.inner.clone(), path, args);
+                Ok((RpRead::new(), CompleteReader::AlreadyComplete(r)))
+            }
             (true, false) => {
-                let r = oio::into_streamable_read(r, 256 * 1024);
-                Ok((rp, CompleteReader::NeedStreamable(r)))
+                let r = FileReader::new(self.inner.clone(), path, args);
+
+                Ok((RpRead::new(), CompleteReader::NeedStreamable(r)))
             }
             _ => {
-                let (offset, size) = match (range.offset(), range.size()) {
-                    (Some(offset), _) => (offset, content_length),
-                    (None, None) => (0, content_length),
-                    (None, Some(size)) => {
-                        // TODO: we can read content range to calculate
-                        // the total content length.
-                        let om = self.inner.stat(path, OpStat::new()).await?.into_metadata();
-                        let total_size = om.content_length();
-                        let (offset, size) = if size > total_size {
-                            (0, total_size)
-                        } else {
-                            (total_size - size, size)
-                        };
-
-                        (offset, size)
-                    }
-                };
-                let r = oio::into_seekable_read_by_range(self.inner.clone(), path, r, offset, size);
+                let r = RangeReader::new(self.inner.clone(), path, args);
 
                 if streamable {
-                    Ok((rp, CompleteReader::NeedSeekable(r)))
+                    Ok((RpRead::new(), CompleteReader::NeedSeekable(r)))
                 } else {
                     let r = oio::into_streamable_read(r, 256 * 1024);
-                    Ok((rp, CompleteReader::NeedBoth(r)))
+                    Ok((RpRead::new(), CompleteReader::NeedBoth(r)))
                 }
             }
         }
     }
 
-    fn complete_blocking_reader(
+    fn complete_blocking_read(
         &self,
         path: &str,
         args: OpRead,
@@ -215,44 +332,23 @@ impl<A: Accessor> CompleteReaderAccessor<A> {
         let seekable = capability.read_can_seek;
         let streamable = capability.read_can_next;
 
-        let range = args.range();
-        let (rp, r) = self.inner.blocking_read(path, args)?;
-        let content_length = rp.metadata().content_length();
-
         match (seekable, streamable) {
-            (true, true) => Ok((rp, CompleteReader::AlreadyComplete(r))),
+            (true, true) => {
+                let r = LazyReader::new(self.inner.clone(), path, args);
+                Ok((RpRead::new(), CompleteReader::AlreadyComplete(r)))
+            }
             (true, false) => {
-                let r = oio::into_streamable_read(r, 256 * 1024);
-                Ok((rp, CompleteReader::NeedStreamable(r)))
+                let r = FileReader::new(self.inner.clone(), path, args);
+                Ok((RpRead::new(), CompleteReader::NeedStreamable(r)))
             }
             _ => {
-                let (offset, size) = match (range.offset(), range.size()) {
-                    (Some(offset), _) => (offset, content_length),
-                    (None, None) => (0, content_length),
-                    (None, Some(size)) => {
-                        // TODO: we can read content range to calculate
-                        // the total content length.
-                        let om = self
-                            .inner
-                            .blocking_stat(path, OpStat::new())?
-                            .into_metadata();
-                        let total_size = om.content_length();
-                        let (offset, size) = if size > total_size {
-                            (0, total_size)
-                        } else {
-                            (total_size - size, size)
-                        };
-
-                        (offset, size)
-                    }
-                };
-                let r = oio::into_seekable_read_by_range(self.inner.clone(), path, r, offset, size);
+                let r = RangeReader::new(self.inner.clone(), path, args);
 
                 if streamable {
-                    Ok((rp, CompleteReader::NeedSeekable(r)))
+                    Ok((RpRead::new(), CompleteReader::NeedSeekable(r)))
                 } else {
                     let r = oio::into_streamable_read(r, 256 * 1024);
-                    Ok((rp, CompleteReader::NeedBoth(r)))
+                    Ok((RpRead::new(), CompleteReader::NeedBoth(r)))
                 }
             }
         }
@@ -262,96 +358,90 @@ impl<A: Accessor> CompleteReaderAccessor<A> {
         &self,
         path: &str,
         args: OpList,
-    ) -> Result<(RpList, CompletePager<A, A::Pager>)> {
+    ) -> Result<(RpList, CompleteLister<A, A::Lister>)> {
         let cap = self.meta.full_capability();
         if !cap.list {
             return Err(self.new_unsupported_error(Operation::List));
         }
 
-        let delimiter = args.delimiter();
+        let recursive = args.recursive();
 
-        if delimiter.is_empty() {
-            return if cap.list_without_delimiter {
+        match (
+            recursive,
+            cap.list_with_recursive,
+            cap.list_without_recursive,
+        ) {
+            // - If service can both list_with_recursive and list_without_recursive
+            // - If recursive is true while services can list_with_recursive
+            // - If recursive is false while services can list_without_recursive
+            (_, true, true) | (true, true, _) | (false, _, true) => {
                 let (rp, p) = self.inner.list(path, args).await?;
-                Ok((rp, CompletePager::AlreadyComplete(p)))
-            } else {
-                let p = into_flat_page(
-                    self.inner.clone(),
-                    path,
-                    args.with_delimiter("/").limit().unwrap_or(1000),
-                );
-                Ok((RpList::default(), CompletePager::NeedFlat(p)))
-            };
+                Ok((rp, CompleteLister::AlreadyComplete(p)))
+            }
+            // If services can't list_with_recursive nor list_without_recursive.
+            //
+            // It should be a service level bug.
+            (_, false, false) => Err(self.new_unsupported_error(Operation::List)),
+            // If recursive is true but service can't list_with_recursive
+            (true, false, true) => {
+                let p = FlatLister::new(self.inner.clone(), path);
+                Ok((RpList::default(), CompleteLister::NeedFlat(p)))
+            }
+            // If recursive is false but service can't list_without_recursive
+            (false, true, false) => {
+                let (_, p) = self.inner.list(path, args.with_recursive(true)).await?;
+                let p = HierarchyLister::new(p, path);
+                Ok((RpList::default(), CompleteLister::NeedHierarchy(p)))
+            }
         }
-
-        if delimiter == "/" {
-            return if cap.list_with_delimiter_slash {
-                let (rp, p) = self.inner.list(path, args).await?;
-                Ok((rp, CompletePager::AlreadyComplete(p)))
-            } else {
-                let (_, p) = self.inner.list(path, args.with_delimiter("")).await?;
-                let p = into_hierarchy_page(p, path);
-                Ok((RpList::default(), CompletePager::NeedHierarchy(p)))
-            };
-        }
-
-        Err(Error::new(
-            ErrorKind::Unsupported,
-            "list with other delimiter is not supported",
-        )
-        .with_context("service", self.meta.scheme())
-        .with_context("delimiter", delimiter))
     }
 
     fn complete_blocking_list(
         &self,
         path: &str,
         args: OpList,
-    ) -> Result<(RpList, CompletePager<A, A::BlockingPager>)> {
+    ) -> Result<(RpList, CompleteLister<A, A::BlockingLister>)> {
         let cap = self.meta.full_capability();
         if !cap.list {
             return Err(self.new_unsupported_error(Operation::BlockingList));
         }
 
-        let delimiter = args.delimiter();
+        let recursive = args.recursive();
 
-        if delimiter.is_empty() {
-            return if cap.list_without_delimiter {
+        match (
+            recursive,
+            cap.list_with_recursive,
+            cap.list_without_recursive,
+        ) {
+            // - If service can both list_with_recursive and list_without_recursive
+            // - If recursive is true while services can list_with_recursive
+            // - If recursive is false while services can list_without_recursive
+            (_, true, true) | (true, true, _) | (false, _, true) => {
                 let (rp, p) = self.inner.blocking_list(path, args)?;
-                Ok((rp, CompletePager::AlreadyComplete(p)))
-            } else {
-                let p = into_flat_page(
-                    self.inner.clone(),
-                    path,
-                    args.with_delimiter("/").limit().unwrap_or(1000),
-                );
-                Ok((RpList::default(), CompletePager::NeedFlat(p)))
-            };
+                Ok((rp, CompleteLister::AlreadyComplete(p)))
+            }
+            // If services can't list_with_recursive nor list_without_recursive.
+            //
+            // It should be a service level bug.
+            (_, false, false) => Err(self.new_unsupported_error(Operation::List)),
+            // If recursive is true but service can't list_with_recursive
+            (true, false, true) => {
+                let p = FlatLister::new(self.inner.clone(), path);
+                Ok((RpList::default(), CompleteLister::NeedFlat(p)))
+            }
+            // If recursive is false but service can't list_without_recursive
+            (false, true, false) => {
+                let (_, p) = self.inner.blocking_list(path, args.with_recursive(true))?;
+                let p: HierarchyLister<<A as Accessor>::BlockingLister> =
+                    HierarchyLister::new(p, path);
+                Ok((RpList::default(), CompleteLister::NeedHierarchy(p)))
+            }
         }
-
-        if delimiter == "/" {
-            return if cap.list_with_delimiter_slash {
-                let (rp, p) = self.inner.blocking_list(path, args)?;
-                Ok((rp, CompletePager::AlreadyComplete(p)))
-            } else {
-                let (_, p) = self.inner.blocking_list(path, args.with_delimiter(""))?;
-                let p: HierarchyPager<<A as Accessor>::BlockingPager> =
-                    into_hierarchy_page(p, path);
-                Ok((RpList::default(), CompletePager::NeedHierarchy(p)))
-            };
-        }
-
-        Err(Error::new(
-            ErrorKind::Unsupported,
-            "list with other delimiter is not supported",
-        )
-        .with_context("service", self.meta.scheme())
-        .with_context("delimiter", delimiter))
     }
 }
 
 #[async_trait]
-impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
+impl<A: Accessor> LayeredAccessor for CompleteAccessor<A> {
     type Inner = A;
     type Reader = CompleteReader<A, A::Reader>;
     type BlockingReader = CompleteReader<A, A::BlockingReader>;
@@ -360,8 +450,8 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
         oio::ExactBufWriter<CompleteWriter<A::Writer>>,
     >;
     type BlockingWriter = CompleteWriter<A::BlockingWriter>;
-    type Pager = CompletePager<A, A::Pager>;
-    type BlockingPager = CompletePager<A, A::BlockingPager>;
+    type Lister = CompleteLister<A, A::Lister>;
+    type BlockingLister = CompleteLister<A, A::BlockingLister>;
 
     fn inner(&self) -> &Self::Inner {
         &self.inner
@@ -374,20 +464,18 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
             cap.read_can_next = true;
             cap.read_can_seek = true;
         }
+        if cap.list && cap.write_can_empty {
+            cap.create_dir = true;
+        }
         meta
     }
 
     async fn create_dir(&self, path: &str, args: OpCreateDir) -> Result<RpCreateDir> {
-        let capability = self.meta.full_capability();
-        if !capability.create_dir {
-            return Err(self.new_unsupported_error(Operation::CreateDir));
-        }
-
-        self.inner().create_dir(path, args).await
+        self.complete_create_dir(path, args).await
     }
 
     async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
-        self.complete_reader(path, args).await
+        self.complete_read(path, args).await
     }
 
     async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
@@ -452,17 +540,7 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
     }
 
     async fn stat(&self, path: &str, args: OpStat) -> Result<RpStat> {
-        let capability = self.meta.full_capability();
-        if !capability.stat {
-            return Err(self.new_unsupported_error(Operation::Stat));
-        }
-
-        self.inner.stat(path, args).await.map(|v| {
-            v.map_metadata(|m| {
-                let bit = m.bit();
-                m.with_bit(bit | Metakey::Complete)
-            })
-        })
+        self.complete_stat(path, args).await
     }
 
     async fn delete(&self, path: &str, args: OpDelete) -> Result<RpDelete> {
@@ -474,7 +552,7 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
         self.inner().delete(path, args).await
     }
 
-    async fn list(&self, path: &str, args: OpList) -> Result<(RpList, Self::Pager)> {
+    async fn list(&self, path: &str, args: OpList) -> Result<(RpList, Self::Lister)> {
         let capability = self.meta.full_capability();
         if !capability.list {
             return Err(self.new_unsupported_error(Operation::List));
@@ -502,16 +580,11 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
     }
 
     fn blocking_create_dir(&self, path: &str, args: OpCreateDir) -> Result<RpCreateDir> {
-        let capability = self.meta.full_capability();
-        if !capability.create_dir || !capability.blocking {
-            return Err(self.new_unsupported_error(Operation::BlockingCreateDir));
-        }
-
-        self.inner().blocking_create_dir(path, args)
+        self.complete_blocking_create_dir(path, args)
     }
 
     fn blocking_read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::BlockingReader)> {
-        self.complete_blocking_reader(path, args)
+        self.complete_blocking_read(path, args)
     }
 
     fn blocking_write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::BlockingWriter)> {
@@ -554,17 +627,7 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
     }
 
     fn blocking_stat(&self, path: &str, args: OpStat) -> Result<RpStat> {
-        let capability = self.meta.full_capability();
-        if !capability.stat || !capability.blocking {
-            return Err(self.new_unsupported_error(Operation::BlockingStat));
-        }
-
-        self.inner.blocking_stat(path, args).map(|v| {
-            v.map_metadata(|m| {
-                let bit = m.bit();
-                m.with_bit(bit | Metakey::Complete)
-            })
-        })
+        self.complete_blocking_stat(path, args)
     }
 
     fn blocking_delete(&self, path: &str, args: OpDelete) -> Result<RpDelete> {
@@ -576,7 +639,7 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
         self.inner().blocking_delete(path, args)
     }
 
-    fn blocking_list(&self, path: &str, args: OpList) -> Result<(RpList, Self::BlockingPager)> {
+    fn blocking_list(&self, path: &str, args: OpList) -> Result<(RpList, Self::BlockingLister)> {
         let capability = self.meta.full_capability();
         if !capability.list || !capability.blocking {
             return Err(self.new_unsupported_error(Operation::BlockingList));
@@ -587,10 +650,10 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
 }
 
 pub enum CompleteReader<A: Accessor, R> {
-    AlreadyComplete(R),
-    NeedSeekable(ByRangeSeekableReader<A, R>),
-    NeedStreamable(StreamableReader<R>),
-    NeedBoth(StreamableReader<ByRangeSeekableReader<A, R>>),
+    AlreadyComplete(LazyReader<A, R>),
+    NeedSeekable(RangeReader<A, R>),
+    NeedStreamable(FileReader<A, R>),
+    NeedBoth(StreamableReader<RangeReader<A, R>>),
 }
 
 impl<A, R> oio::Read for CompleteReader<A, R>
@@ -598,6 +661,7 @@ where
     A: Accessor<Reader = R>,
     R: oio::Read,
 {
+    #[inline]
     fn poll_read(&mut self, cx: &mut Context<'_>, buf: &mut [u8]) -> Poll<Result<usize>> {
         use CompleteReader::*;
 
@@ -671,36 +735,36 @@ where
     }
 }
 
-pub enum CompletePager<A: Accessor, P> {
+pub enum CompleteLister<A: Accessor, P> {
     AlreadyComplete(P),
-    NeedFlat(FlatPager<Arc<A>, P>),
-    NeedHierarchy(HierarchyPager<P>),
+    NeedFlat(FlatLister<Arc<A>, P>),
+    NeedHierarchy(HierarchyLister<P>),
 }
 
 #[async_trait]
-impl<A, P> oio::Page for CompletePager<A, P>
+impl<A, P> oio::List for CompleteLister<A, P>
 where
-    A: Accessor<Pager = P>,
-    P: oio::Page,
+    A: Accessor<Lister = P>,
+    P: oio::List,
 {
-    async fn next(&mut self) -> Result<Option<Vec<Entry>>> {
-        use CompletePager::*;
+    fn poll_next(&mut self, cx: &mut Context<'_>) -> Poll<Result<Option<oio::Entry>>> {
+        use CompleteLister::*;
 
         match self {
-            AlreadyComplete(p) => p.next().await,
-            NeedFlat(p) => p.next().await,
-            NeedHierarchy(p) => p.next().await,
+            AlreadyComplete(p) => p.poll_next(cx),
+            NeedFlat(p) => p.poll_next(cx),
+            NeedHierarchy(p) => p.poll_next(cx),
         }
     }
 }
 
-impl<A, P> oio::BlockingPage for CompletePager<A, P>
+impl<A, P> oio::BlockingList for CompleteLister<A, P>
 where
-    A: Accessor<BlockingPager = P>,
-    P: oio::BlockingPage,
+    A: Accessor<BlockingLister = P>,
+    P: oio::BlockingList,
 {
-    fn next(&mut self) -> Result<Option<Vec<Entry>>> {
-        use CompletePager::*;
+    fn next(&mut self) -> Result<Option<oio::Entry>> {
+        use CompleteLister::*;
 
         match self {
             AlreadyComplete(p) => p.next(),
@@ -814,8 +878,8 @@ mod tests {
         type BlockingReader = oio::BlockingReader;
         type Writer = oio::Writer;
         type BlockingWriter = oio::BlockingWriter;
-        type Pager = oio::Pager;
-        type BlockingPager = oio::BlockingPager;
+        type Lister = oio::Lister;
+        type BlockingLister = oio::BlockingLister;
 
         fn info(&self) -> AccessorInfo {
             let mut info = AccessorInfo::default();
@@ -829,7 +893,7 @@ mod tests {
         }
 
         async fn read(&self, _: &str, _: OpRead) -> Result<(RpRead, Self::Reader)> {
-            Ok((RpRead::new(0), Box::new(())))
+            Ok((RpRead::new(), Box::new(oio::Cursor::new())))
         }
 
         async fn write(&self, _: &str, _: OpWrite) -> Result<(RpWrite, Self::Writer)> {
@@ -852,7 +916,7 @@ mod tests {
             Ok(RpDelete {})
         }
 
-        async fn list(&self, _: &str, _: OpList) -> Result<(RpList, Self::Pager)> {
+        async fn list(&self, _: &str, _: OpList) -> Result<(RpList, Self::Lister)> {
             Ok((RpList {}, Box::new(())))
         }
 
@@ -880,6 +944,7 @@ mod tests {
 
         let op = new_test_operator(Capability {
             read: true,
+            stat: true,
             ..Default::default()
         });
         let res = op.read("path").await;
@@ -985,6 +1050,7 @@ mod tests {
 
         let op = new_test_operator(Capability {
             list: true,
+            list_with_recursive: true,
             ..Default::default()
         });
         let res = op.list("path/").await;
