@@ -218,8 +218,8 @@ impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
     type BlockingReader = LoggingReader<A::BlockingReader>;
     type Writer = LoggingWriter<A::Writer>;
     type BlockingWriter = LoggingWriter<A::BlockingWriter>;
-    type Pager = LoggingPager<A::Pager>;
-    type BlockingPager = LoggingPager<A::BlockingPager>;
+    type Lister = LoggingLister<A::Lister>;
+    type BlockingLister = LoggingLister<A::BlockingLister>;
 
     fn inner(&self) -> &Self::Inner {
         &self.inner
@@ -525,7 +525,7 @@ impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
             .await
     }
 
-    async fn list(&self, path: &str, args: OpList) -> Result<(RpList, Self::Pager)> {
+    async fn list(&self, path: &str, args: OpList) -> Result<(RpList, Self::Lister)> {
         debug!(
             target: LOGGING_TARGET,
             "service={} operation={} path={} -> started",
@@ -545,7 +545,7 @@ impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
                         Operation::List,
                         path
                     );
-                    let streamer = LoggingPager::new(self.ctx.clone(), path, Operation::List, v);
+                    let streamer = LoggingLister::new(self.ctx.clone(), path, Operation::List, v);
                     Ok((rp, streamer))
                 }
                 Err(err) => {
@@ -914,7 +914,7 @@ impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
             })
     }
 
-    fn blocking_list(&self, path: &str, args: OpList) -> Result<(RpList, Self::BlockingPager)> {
+    fn blocking_list(&self, path: &str, args: OpList) -> Result<(RpList, Self::BlockingLister)> {
         debug!(
             target: LOGGING_TARGET,
             "service={} operation={} path={} -> started",
@@ -933,7 +933,7 @@ impl<A: Accessor> LayeredAccessor for LoggingAccessor<A> {
                     Operation::BlockingList,
                     path
                 );
-                let li = LoggingPager::new(self.ctx.clone(), path, Operation::BlockingList, v);
+                let li = LoggingLister::new(self.ctx.clone(), path, Operation::BlockingList, v);
                 (rp, li)
             })
             .map_err(|err| {
@@ -991,17 +991,20 @@ impl<R> Drop for LoggingReader<R> {
 
 impl<R: oio::Read> oio::Read for LoggingReader<R> {
     fn poll_read(&mut self, cx: &mut Context<'_>, buf: &mut [u8]) -> Poll<Result<usize>> {
+        let buf_size = buf.len();
+
         match self.inner.poll_read(cx, buf) {
             Poll::Ready(res) => match res {
                 Ok(n) => {
                     self.read += n as u64;
                     trace!(
                         target: LOGGING_TARGET,
-                        "service={} operation={} path={} read={} -> data read {}B ",
+                        "service={} operation={} path={} read={} -> buf size: {}B, read {}B ",
                         self.ctx.scheme,
                         ReadOperation::Read,
                         self.path,
                         self.read,
+                        buf_size,
                         n
                     );
                     Poll::Ready(Ok(n))
@@ -1011,7 +1014,7 @@ impl<R: oio::Read> oio::Read for LoggingReader<R> {
                         log!(
                             target: LOGGING_TARGET,
                             lvl,
-                            "service={} operation={} path={} read={} -> data read failed: {}",
+                            "service={} operation={} path={} read={} -> read failed: {}",
                             self.ctx.scheme,
                             ReadOperation::Read,
                             self.path,
@@ -1025,11 +1028,12 @@ impl<R: oio::Read> oio::Read for LoggingReader<R> {
             Poll::Pending => {
                 trace!(
                     target: LOGGING_TARGET,
-                    "service={} operation={} path={} read={} -> data read pending",
+                    "service={} operation={} path={} read={} -> buf size: {}B, read pending",
                     self.ctx.scheme,
                     ReadOperation::Read,
                     self.path,
-                    self.read
+                    self.read,
+                    buf_size
                 );
                 Poll::Pending
             }
@@ -1042,7 +1046,7 @@ impl<R: oio::Read> oio::Read for LoggingReader<R> {
                 Ok(n) => {
                     trace!(
                         target: LOGGING_TARGET,
-                        "service={} operation={} path={} read={} -> data seek to offset {n}",
+                        "service={} operation={} path={} read={} -> seek to {pos:?}, current offset {n}",
                         self.ctx.scheme,
                         ReadOperation::Seek,
                         self.path,
@@ -1055,7 +1059,7 @@ impl<R: oio::Read> oio::Read for LoggingReader<R> {
                         log!(
                             target: LOGGING_TARGET,
                             lvl,
-                            "service={} operation={} path={} read={} -> data read failed: {}",
+                            "service={} operation={} path={} read={} -> seek to {pos:?} failed: {}",
                             self.ctx.scheme,
                             ReadOperation::Seek,
                             self.path,
@@ -1069,7 +1073,7 @@ impl<R: oio::Read> oio::Read for LoggingReader<R> {
             Poll::Pending => {
                 trace!(
                     target: LOGGING_TARGET,
-                    "service={} operation={} path={} read={} -> data seek pending",
+                    "service={} operation={} path={} read={} -> seek to {pos:?} pending",
                     self.ctx.scheme,
                     ReadOperation::Seek,
                     self.path,
@@ -1087,7 +1091,7 @@ impl<R: oio::Read> oio::Read for LoggingReader<R> {
                     self.read += bs.len() as u64;
                     trace!(
                         target: LOGGING_TARGET,
-                        "service={} operation={} path={} read={} -> data read {}B",
+                        "service={} operation={} path={} read={} -> next returns {}B",
                         self.ctx.scheme,
                         ReadOperation::Next,
                         self.path,
@@ -1101,7 +1105,7 @@ impl<R: oio::Read> oio::Read for LoggingReader<R> {
                         log!(
                             target: LOGGING_TARGET,
                             lvl,
-                            "service={} operation={} path={} read={} -> data read failed: {}",
+                            "service={} operation={} path={} read={} -> next failed: {}",
                             self.ctx.scheme,
                             ReadOperation::Next,
                             self.path,
@@ -1111,12 +1115,22 @@ impl<R: oio::Read> oio::Read for LoggingReader<R> {
                     }
                     Poll::Ready(Some(Err(err)))
                 }
-                None => Poll::Ready(None),
+                None => {
+                    trace!(
+                        target: LOGGING_TARGET,
+                        "service={} operation={} path={} read={} -> next returns None",
+                        self.ctx.scheme,
+                        ReadOperation::Next,
+                        self.path,
+                        self.read,
+                    );
+                    Poll::Ready(None)
+                }
             },
             Poll::Pending => {
                 trace!(
                     target: LOGGING_TARGET,
-                    "service={} operation={} path={} read={} -> data read pending",
+                    "service={} operation={} path={} read={} -> next returns pending",
                     self.ctx.scheme,
                     ReadOperation::Next,
                     self.path,
@@ -1417,7 +1431,7 @@ impl<W: oio::BlockingWrite> oio::BlockingWrite for LoggingWriter<W> {
     }
 }
 
-pub struct LoggingPager<P> {
+pub struct LoggingLister<P> {
     ctx: LoggingContext,
     path: String,
     op: Operation,
@@ -1426,7 +1440,7 @@ pub struct LoggingPager<P> {
     inner: P,
 }
 
-impl<P> LoggingPager<P> {
+impl<P> LoggingLister<P> {
     fn new(ctx: LoggingContext, path: &str, op: Operation, inner: P) -> Self {
         Self {
             ctx,
@@ -1438,7 +1452,7 @@ impl<P> LoggingPager<P> {
     }
 }
 
-impl<P> Drop for LoggingPager<P> {
+impl<P> Drop for LoggingLister<P> {
     fn drop(&mut self) {
         if self.finished {
             debug!(
@@ -1461,19 +1475,19 @@ impl<P> Drop for LoggingPager<P> {
 }
 
 #[async_trait]
-impl<P: oio::Page> oio::Page for LoggingPager<P> {
-    async fn next(&mut self) -> Result<Option<Vec<oio::Entry>>> {
-        let res = self.inner.next().await;
+impl<P: oio::List> oio::List for LoggingLister<P> {
+    fn poll_next(&mut self, cx: &mut Context<'_>) -> Poll<Result<Option<oio::Entry>>> {
+        let res = ready!(self.inner.poll_next(cx));
 
         match &res {
-            Ok(Some(des)) => {
+            Ok(Some(de)) => {
                 debug!(
                     target: LOGGING_TARGET,
-                    "service={} operation={} path={} -> listed {} entries",
+                    "service={} operation={} path={} -> listed entry: {}",
                     self.ctx.scheme,
                     self.op,
                     self.path,
-                    des.len(),
+                    de.path(),
                 );
             }
             Ok(None) => {
@@ -1501,23 +1515,23 @@ impl<P: oio::Page> oio::Page for LoggingPager<P> {
             }
         };
 
-        res
+        Poll::Ready(res)
     }
 }
 
-impl<P: oio::BlockingPage> oio::BlockingPage for LoggingPager<P> {
-    fn next(&mut self) -> Result<Option<Vec<oio::Entry>>> {
+impl<P: oio::BlockingList> oio::BlockingList for LoggingLister<P> {
+    fn next(&mut self) -> Result<Option<oio::Entry>> {
         let res = self.inner.next();
 
         match &res {
             Ok(Some(des)) => {
                 debug!(
                     target: LOGGING_TARGET,
-                    "service={} operation={} path={} -> got {} entries",
+                    "service={} operation={} path={} -> listed entry: {}",
                     self.ctx.scheme,
                     self.op,
                     self.path,
-                    des.len(),
+                    des.path(),
                 );
             }
             Ok(None) => {
