@@ -97,32 +97,36 @@ impl<T: Default> From<PresignedRequest> for Request<T> {
 }
 
 /// Reply for `read` operation.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Default)]
 pub struct RpRead {
-    meta: Metadata,
+    /// Size is the size of the reader returned by this read operation.
+    ///
+    /// - `Some(size)` means the reader has at most size bytes.
+    /// - `None` means the reader has unknown size.
+    ///
+    /// It's ok to leave size as empty, but it's recommended to set size if possible. We will use
+    /// this size as hint to do some optimization like avoid an extra stat or read.
+    size: Option<u64>,
 }
 
 impl RpRead {
     /// Create a new reply for `read`.
-    pub fn new(content_length: u64) -> Self {
-        RpRead {
-            meta: Metadata::new(EntryMode::FILE).with_content_length(content_length),
-        }
+    pub fn new() -> Self {
+        RpRead::default()
     }
 
-    /// Create reply read with existing metadata.
-    pub fn with_metadata(meta: Metadata) -> Self {
-        RpRead { meta }
+    /// Got the size of the reader returned by this read operation.
+    ///
+    /// - `Some(size)` means the reader has at most size bytes.
+    /// - `None` means the reader has unknown size.
+    pub fn size(&self) -> Option<u64> {
+        self.size
     }
 
-    /// Get a ref of metadata.
-    pub fn metadata(&self) -> &Metadata {
-        &self.meta
-    }
-
-    /// Consume reply to get the meta.
-    pub fn into_metadata(self) -> Metadata {
-        self.meta
+    /// Set the size of the reader returned by this read operation.
+    pub fn with_size(mut self, size: Option<u64>) -> Self {
+        self.size = size;
+        self
     }
 }
 
