@@ -18,7 +18,6 @@
 use std::sync::Arc;
 
 use async_trait::async_trait;
-use bytes::Buf;
 use http::StatusCode;
 
 use super::core::*;
@@ -26,6 +25,9 @@ use super::error::parse_error;
 use crate::raw::*;
 use crate::*;
 
+pub type CosWriters =
+    oio::TwoWaysWriter<oio::MultipartUploadWriter<CosWriter>, oio::AppendObjectWriter<CosWriter>>;
+
 pub struct CosWriter {
     core: Arc<CosCore>,
 
@@ -34,29 +36,21 @@ pub struct CosWriter {
 }
 
 impl CosWriter {
-    pub fn new(core: Arc<CosCore>, path: &str, op: OpWrite) -> oio::MultipartUploadWriter<Self> {
-        let write_min_size = core.write_min_size;
-        let total_size = op.content_length();
-        let cos_writer = CosWriter {
+    pub fn new(core: Arc<CosCore>, path: &str, op: OpWrite) -> Self {
+        CosWriter {
             core,
             path: path.to_string(),
             op,
-        };
-        oio::MultipartUploadWriter::new(cos_writer, total_size).with_write_min_size(write_min_size)
+        }
     }
 }
 
 #[async_trait]
 impl oio::MultipartUploadWrite for CosWriter {
     async fn write_once(&self, size: u64, body: AsyncBody) -> Result<()> {
-        let mut req = self.core.cos_put_object_request(
-            &self.path,
-            Some(size),
-            self.op.content_type(),
-            self.op.content_disposition(),
-            self.op.cache_control(),
-            body,
-        )?;
+        let mut req = self
+            .core
+            .cos_put_object_request(&self.path, Some(size), &self.op, body)?;
 
         self.core.sign(&mut req).await?;
 
@@ -76,12 +70,7 @@ impl oio::MultipartUploadWrite for CosWriter {
     async fn initiate_part(&self) -> Result<String> {
         let resp = self
             .core
-            .cos_initiate_multipart_upload(
-                &self.path,
-                self.op.content_type(),
-                self.op.content_disposition(),
-                self.op.cache_control(),
-            )
+            .cos_initiate_multipart_upload(&self.path, &self.op)
             .await?;
 
         let status = resp.status();
@@ -91,7 +80,8 @@ impl oio::MultipartUploadWrite for CosWriter {
                 let bs = resp.into_body().bytes().await?;
 
                 let result: InitiateMultipartUploadResult =
-                    quick_xml::de::from_reader(bs.reader()).map_err(new_xml_deserialize_error)?;
+                    quick_xml::de::from_reader(bytes::Buf::reader(bs))
+                        .map_err(new_xml_deserialize_error)?;
 
                 Ok(result.upload_id)
             }
@@ -181,3 +171,45 @@ impl oio::MultipartUploadWrite for CosWriter {
         }
     }
 }
+
+#[async_trait]
+impl oio::AppendObjectWrite for CosWriter {
+    async fn offset(&self) -> Result<u64> {
+        let resp = self
+            .core
+            .cos_head_object(&self.path, &OpStat::default())
+            .await?;
+
+        let status = resp.status();
+        match status {
+            StatusCode::OK => {
+                let content_length = parse_content_length(resp.headers())?.ok_or_else(|| {
+                    Error::new(
+                        ErrorKind::Unexpected,
+                        "Content-Length not present in returning response",
+                    )
+                })?;
+                Ok(content_length)
+            }
+            StatusCode::NOT_FOUND => Ok(0),
+            _ => Err(parse_error(resp).await?),
+        }
+    }
+
+    async fn append(&self, offset: u64, size: u64, body: AsyncBody) -> Result<()> {
+        let mut req = self
+            .core
+            .cos_append_object_request(&self.path, offset, size, &self.op, body)?;
+
+        self.core.sign(&mut req).await?;
+
+        let resp = self.core.send(req).await?;
+
+        let status = resp.status();
+
+        match status {
+            StatusCode::OK => Ok(()),
+            _ => Err(parse_error(resp).await?),
+        }
+    }
+}
