@@ -27,11 +27,10 @@ use serde_json::json;
 
 use super::core::GdriveCore;
 use super::error::parse_error;
-use super::pager::GdrivePager;
+use super::lister::GdriveLister;
 use super::writer::GdriveWriter;
 use crate::raw::*;
 use crate::services::gdrive::core::GdriveFile;
-use crate::services::gdrive::core::GdriveFileList;
 use crate::types::Result;
 use crate::*;
 
@@ -46,8 +45,8 @@ impl Accessor for GdriveBackend {
     type BlockingReader = ();
     type Writer = oio::OneShotWriter<GdriveWriter>;
     type BlockingWriter = ();
-    type Pager = GdrivePager;
-    type BlockingPager = ();
+    type Lister = oio::PageLister<GdriveLister>;
+    type BlockingLister = ();
 
     fn info(&self) -> AccessorInfo {
         let mut ma = AccessorInfo::default();
@@ -59,8 +58,7 @@ impl Accessor for GdriveBackend {
                 read: true,
 
                 list: true,
-
-                list_with_delimiter_slash: true,
+                list_without_recursive: true,
 
                 write: true,
 
@@ -79,93 +77,46 @@ impl Accessor for GdriveBackend {
     }
 
     async fn stat(&self, path: &str, _args: OpStat) -> Result<RpStat> {
-        // Stat root always returns a DIR.
-        if path == "/" {
-            return Ok(RpStat::new(Metadata::new(EntryMode::DIR)));
-        }
-
         let resp = self.core.gdrive_stat(path).await?;
 
-        let status = resp.status();
-
-        match status {
-            StatusCode::OK => {
-                let meta = self.parse_metadata(resp.into_body().bytes().await?)?;
-                Ok(RpStat::new(meta))
-            }
-            _ => Err(parse_error(resp).await?),
+        if resp.status() != StatusCode::OK {
+            return Err(parse_error(resp).await?);
         }
+
+        let meta = self.parse_metadata(resp.into_body().bytes().await?)?;
+        Ok(RpStat::new(meta))
     }
 
     async fn create_dir(&self, path: &str, _args: OpCreateDir) -> Result<RpCreateDir> {
         let parent = self.core.ensure_parent_path(path).await?;
 
-        let path = path.split('/').filter(|&x| !x.is_empty()).last().unwrap();
+        // Make sure `/` has been trimmed.
+        let path = get_basename(path).trim_end_matches('/');
 
         // As Google Drive allows files have the same name, we need to check if the folder exists.
-        let resp = self.core.gdrive_search_folder(path, &parent).await?;
-        let status = resp.status();
+        let folder_id = self.core.gdrive_search_folder(&parent, path).await?;
 
-        match status {
-            StatusCode::OK => {
-                let body = resp.into_body().bytes().await?;
-                let meta = serde_json::from_slice::<GdriveFileList>(&body)
-                    .map_err(new_json_deserialize_error)?;
+        let id = if let Some(id) = folder_id {
+            id
+        } else {
+            self.core.gdrive_create_folder(&parent, path).await?
+        };
 
-                if !meta.files.is_empty() {
-                    let mut cache = self.core.path_cache.lock().await;
+        let mut cache = self.core.path_cache.lock().await;
+        cache.insert(build_abs_path(&self.core.root, path), id);
 
-                    cache.insert(
-                        build_abs_path(&self.core.root, path),
-                        meta.files[0].id.clone(),
-                    );
-
-                    return Ok(RpCreateDir::default());
-                }
-            }
-            _ => return Err(parse_error(resp).await?),
-        }
-
-        let resp = self.core.gdrive_create_folder(path, Some(parent)).await?;
-
-        let status = resp.status();
-
-        match status {
-            StatusCode::OK => {
-                let body = resp.into_body().bytes().await?;
-                let meta = serde_json::from_slice::<GdriveFile>(&body)
-                    .map_err(new_json_deserialize_error)?;
-
-                let mut cache = self.core.path_cache.lock().await;
-
-                cache.insert(build_abs_path(&self.core.root, path), meta.id.clone());
-
-                Ok(RpCreateDir::default())
-            }
-            _ => Err(parse_error(resp).await?),
-        }
+        Ok(RpCreateDir::default())
     }
 
     async fn read(&self, path: &str, _args: OpRead) -> Result<(RpRead, Self::Reader)> {
-        // We need to request for metadata and body separately here.
-        // Request for metadata first to check if the file exists.
-        let resp = self.core.gdrive_stat(path).await?;
+        let resp = self.core.gdrive_get(path).await?;
 
         let status = resp.status();
 
         match status {
             StatusCode::OK => {
-                let body = resp.into_body().bytes().await?;
-                let meta = self.parse_metadata(body)?;
-
-                let resp = self.core.gdrive_get(path).await?;
-
-                let status = resp.status();
-
-                match status {
-                    StatusCode::OK => Ok((RpRead::with_metadata(meta), resp.into_body())),
-                    _ => Err(parse_error(resp).await?),
-                }
+                let size = parse_content_length(resp.headers())?;
+                Ok((RpRead::new().with_size(size), resp.into_body()))
             }
             _ => Err(parse_error(resp).await?),
         }
@@ -233,7 +184,7 @@ impl Accessor for GdriveBackend {
             let status = resp.status();
 
             match status {
-                StatusCode::NO_CONTENT => {
+                StatusCode::NO_CONTENT | StatusCode::NOT_FOUND => {
                     let mut cache = self.core.path_cache.lock().await;
 
                     cache.remove(&build_abs_path(&self.core.root, path));
@@ -252,11 +203,9 @@ impl Accessor for GdriveBackend {
         }
     }
 
-    async fn list(&self, path: &str, _args: OpList) -> Result<(RpList, Self::Pager)> {
-        Ok((
-            RpList::default(),
-            GdrivePager::new(path.into(), self.core.clone()),
-        ))
+    async fn list(&self, path: &str, _args: OpList) -> Result<(RpList, Self::Lister)> {
+        let l = GdriveLister::new(path.into(), self.core.clone());
+        Ok((RpList::default(), oio::PageLister::new(l)))
     }
 
     async fn copy(&self, from: &str, to: &str, _args: OpCopy) -> Result<RpCopy> {
