@@ -23,32 +23,56 @@ use http::StatusCode;
 
 use super::core::GdriveCore;
 use super::error::parse_error;
+use crate::raw::oio::WriteBuf;
 use crate::raw::*;
 use crate::*;
 
 pub struct GdriveWriter {
     core: Arc<GdriveCore>,
-    op: OpWrite,
+
     path: String,
+
+    file_id: Option<String>,
 }
 
 impl GdriveWriter {
-    pub fn new(core: Arc<GdriveCore>, op: OpWrite, path: String) -> Self {
-        GdriveWriter { core, op, path }
-    }
-}
+    pub fn new(core: Arc<GdriveCore>, path: String, file_id: Option<String>) -> Self {
+        GdriveWriter {
+            core,
+            path,
 
-#[async_trait]
-impl oio::Write for GdriveWriter {
-    async fn write(&mut self, bs: Bytes) -> Result<()> {
+            file_id,
+        }
+    }
+
+    /// Write a single chunk of data to the object.
+    ///
+    /// This is used for small objects.
+    /// And should overwrite the object if it already exists.
+    pub async fn write_create(&self, size: u64, body: Bytes) -> Result<()> {
         let resp = self
             .core
-            .gdrive_update(
-                &self.path,
-                Some(bs.len()),
-                self.op.content_type(),
-                AsyncBody::Bytes(bs),
-            )
+            .gdrive_upload_simple_request(&self.path, size, body)
+            .await?;
+
+        let status = resp.status();
+
+        match status {
+            StatusCode::OK | StatusCode::CREATED => {
+                resp.into_body().consume().await?;
+                Ok(())
+            }
+            _ => Err(parse_error(resp).await?),
+        }
+    }
+
+    pub async fn write_overwrite(&self, size: u64, body: Bytes) -> Result<()> {
+        let file_id = self.file_id.as_ref().ok_or_else(|| {
+            Error::new(ErrorKind::Unexpected, "file_id is required for overwrite")
+        })?;
+        let resp = self
+            .core
+            .gdrive_upload_overwrite_simple_request(file_id, size, body)
             .await?;
 
         let status = resp.status();
@@ -61,19 +85,19 @@ impl oio::Write for GdriveWriter {
             _ => Err(parse_error(resp).await?),
         }
     }
+}
 
-    async fn sink(&mut self, _size: u64, _s: oio::Streamer) -> Result<()> {
-        Err(Error::new(
-            ErrorKind::Unsupported,
-            "Write::sink is not supported",
-        ))
-    }
+#[async_trait]
+impl oio::OneShotWrite for GdriveWriter {
+    async fn write_once(&self, bs: &dyn WriteBuf) -> Result<()> {
+        let bs = bs.bytes(bs.remaining());
+        let size = bs.len();
+        if self.file_id.is_none() {
+            self.write_create(size as u64, bs).await?;
+        } else {
+            self.write_overwrite(size as u64, bs).await?;
+        }
 
-    async fn abort(&mut self) -> Result<()> {
-        Ok(())
-    }
-
-    async fn close(&mut self) -> Result<()> {
         Ok(())
     }
 }
