@@ -81,96 +81,11 @@ fn value_or_env(
 }
 
 /// GitHub Action Cache Services support.
-///
-/// # Capabilities
-///
-/// This service can be used to:
-///
-/// - [x] stat
-/// - [x] read
-/// - [x] write
-/// - [x] create_dir
-/// - [x] delete
-/// - [x] copy
-/// - [ ] rename
-/// - [ ] list
-/// - [ ] scan
-/// - [ ] presign
-/// - [ ] blocking
-/// # Notes
-///
-/// This service is mainly provided by github actions.
-///
-/// Refer to [Caching dependencies to speed up workflows](https://docs.github.com/en/actions/using-workflows/caching-dependencies-to-speed-up-workflows) for more information.
-///
-/// To make this service work as expected, please make sure to either call `endpoint` and `token` to
-/// configure the URL and credentials, or that the following environment has been setup correctly:
-///
-/// - `ACTIONS_CACHE_URL`
-/// - `ACTIONS_RUNTIME_TOKEN`
-///
-/// They can be exposed by following action:
-///
-/// ```yaml
-/// - name: Configure Cache Env
-///   uses: actions/github-script@v6
-///   with:
-///     script: |
-///       core.exportVariable('ACTIONS_CACHE_URL', process.env.ACTIONS_CACHE_URL || '');
-///       core.exportVariable('ACTIONS_RUNTIME_TOKEN', process.env.ACTIONS_RUNTIME_TOKEN || '');
-/// ```
-///
-/// To make `delete` work as expected, `GITHUB_TOKEN` should also be set via:
-///
-/// ```yaml
-/// env:
-///   GITHUB_TOKEN: ${{ secrets.GITHUB_TOKEN }}
-/// ```
-///
-/// # Limitations
-///
-/// Unlike other services, ghac doesn't support create empty files.
-/// We provide a `enable_create_simulation()` to support this operation but may result unexpected side effects.
-///
-/// Also, `ghac` is a cache service which means the data store inside could
-/// be automatically evicted at any time.
-///
-/// # Configuration
-///
-/// - `root`: Set the work dir for backend.
-///
-/// Refer to [`GhacBuilder`]'s public API docs for more information.
-///
-/// # Example
-///
-/// ## Via Builder
-///
-/// ```no_run
-/// use std::sync::Arc;
-///
-/// use anyhow::Result;
-/// use opendal::services::Ghac;
-/// use opendal::Operator;
-///
-/// #[tokio::main]
-/// async fn main() -> Result<()> {
-///     // Create ghac backend builder.
-///     let mut builder = Ghac::default();
-///     // Set the root for ghac, all operations will happen under this root.
-///     //
-///     // NOTE: the root must be absolute path.
-///     builder.root("/path/to/dir");
-///
-///     let op: Operator = Operator::new(builder)?.finish();
-///
-///     Ok(())
-/// }
-/// ```
+#[doc = include_str!("docs.md")]
 #[derive(Debug, Default)]
 pub struct GhacBuilder {
     root: Option<String>,
     version: Option<String>,
-    enable_create_simulation: bool,
     endpoint: Option<String>,
     runtime_token: Option<String>,
 
@@ -201,18 +116,6 @@ impl GhacBuilder {
         self
     }
 
-    /// Enable create simulation for ghac service.
-    ///
-    /// ghac service doesn't support create empty files. By enabling
-    /// create simulation, we will create a 1 byte file to represent
-    /// empty file.
-    ///
-    /// As a side effect, we can't create file with only 1 byte anymore.
-    pub fn enable_create_simulation(&mut self) -> &mut Self {
-        self.enable_create_simulation = true;
-        self
-    }
-
     /// Set the endpoint for ghac service.
     ///
     /// For example, this is provided as the `ACTIONS_CACHE_URL` environment variable by the GHA runner.
@@ -259,9 +162,6 @@ impl Builder for GhacBuilder {
 
         map.get("root").map(|v| builder.root(v));
         map.get("version").map(|v| builder.version(v));
-        map.get("enable_create_simulation")
-            .filter(|v| *v == "on" || *v == "true")
-            .map(|_| builder.enable_create_simulation());
 
         builder
     }
@@ -283,7 +183,6 @@ impl Builder for GhacBuilder {
 
         let backend = GhacBackend {
             root,
-            enable_create_simulation: self.enable_create_simulation,
 
             cache_url: value_or_env(self.endpoint.take(), ACTIONS_CACHE_URL, "Builder::build")?,
             catch_token: value_or_env(
@@ -313,7 +212,6 @@ impl Builder for GhacBuilder {
 pub struct GhacBackend {
     // root should end with "/"
     root: String,
-    enable_create_simulation: bool,
 
     cache_url: String,
     catch_token: String,
@@ -332,8 +230,8 @@ impl Accessor for GhacBackend {
     type BlockingReader = ();
     type Writer = GhacWriter;
     type BlockingWriter = ();
-    type Pager = ();
-    type BlockingPager = ();
+    type Lister = ();
+    type BlockingLister = ();
 
     fn info(&self) -> AccessorInfo {
         let mut am = AccessorInfo::default();
@@ -349,7 +247,6 @@ impl Accessor for GhacBackend {
 
                 write: true,
                 write_can_multi: true,
-                create_dir: true,
                 delete: true,
 
                 ..Default::default()
@@ -357,64 +254,6 @@ impl Accessor for GhacBackend {
         am
     }
 
-    async fn create_dir(&self, path: &str, _: OpCreateDir) -> Result<RpCreateDir> {
-        // ignore creation of dir.
-        if path.ends_with('/') {
-            return Ok(RpCreateDir::default());
-        }
-        if !self.enable_create_simulation {
-            return Err(Error::new(
-                ErrorKind::Unsupported,
-                "ghac service doesn't support create empty file",
-            ));
-        }
-
-        let req = self.ghac_reserve(path).await?;
-
-        let resp = self.client.send(req).await?;
-
-        let cache_id = if resp.status().is_success() {
-            let slc = resp.into_body().bytes().await?;
-            let reserve_resp: GhacReserveResponse =
-                serde_json::from_slice(&slc).map_err(new_json_deserialize_error)?;
-            reserve_resp.cache_id
-        } else if resp.status().as_u16() == StatusCode::CONFLICT {
-            // If the file is already exist, just return Ok.
-            return Ok(RpCreateDir::default());
-        } else {
-            return Err(parse_error(resp)
-                .await
-                .map(|err| err.with_operation("Backend::ghac_reserve"))?);
-        };
-
-        // Write only 1 byte to allow create.
-        let req = self
-            .ghac_upload(cache_id, 0, 1, AsyncBody::Bytes(Bytes::from_static(&[0])))
-            .await?;
-
-        let resp = self.client.send(req).await?;
-
-        if resp.status().is_success() {
-            resp.into_body().consume().await?;
-        } else {
-            return Err(parse_error(resp)
-                .await
-                .map(|err| err.with_operation("Backend::ghac_upload"))?);
-        }
-
-        let req = self.ghac_commit(cache_id, 1).await?;
-        let resp = self.client.send(req).await?;
-
-        if resp.status().is_success() {
-            resp.into_body().consume().await?;
-            Ok(RpCreateDir::default())
-        } else {
-            Err(parse_error(resp)
-                .await
-                .map(|err| err.with_operation("Backend::ghac_commit"))?)
-        }
-    }
-
     async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
         let req = self.ghac_query(path).await?;
 
@@ -435,9 +274,10 @@ impl Accessor for GhacBackend {
         let status = resp.status();
         match status {
             StatusCode::OK | StatusCode::PARTIAL_CONTENT => {
-                let meta = parse_into_metadata(path, resp.headers())?;
-                Ok((RpRead::with_metadata(meta), resp.into_body()))
+                let size = parse_content_length(resp.headers())?;
+                Ok((RpRead::new().with_size(size), resp.into_body()))
             }
+            StatusCode::RANGE_NOT_SATISFIABLE => Ok((RpRead::new(), IncomingAsyncBody::empty())),
             _ => Err(parse_error(resp).await?),
         }
     }
@@ -462,11 +302,6 @@ impl Accessor for GhacBackend {
     }
 
     async fn stat(&self, path: &str, _: OpStat) -> Result<RpStat> {
-        // Stat root always returns a DIR.
-        if path == "/" {
-            return Ok(RpStat::new(Metadata::new(EntryMode::DIR)));
-        }
-
         let req = self.ghac_query(path).await?;
 
         let resp = self.client.send(req).await?;
@@ -476,8 +311,6 @@ impl Accessor for GhacBackend {
             let query_resp: GhacQueryResponse =
                 serde_json::from_slice(&slc).map_err(new_json_deserialize_error)?;
             query_resp.archive_location
-        } else if resp.status() == StatusCode::NO_CONTENT && path.ends_with('/') {
-            return Ok(RpStat::new(Metadata::new(EntryMode::DIR)));
         } else {
             return Err(parse_error(resp).await?);
         };
@@ -488,12 +321,7 @@ impl Accessor for GhacBackend {
         let status = resp.status();
         match status {
             StatusCode::OK => {
-                let mut meta = parse_into_metadata(path, resp.headers())?;
-
-                // Hack for enable_create_simulation.
-                if self.enable_create_simulation && meta.content_length_raw() == Some(1) {
-                    meta.set_content_length(0);
-                }
+                let meta = parse_into_metadata(path, resp.headers())?;
 
                 Ok(RpStat::new(meta))
             }
