@@ -18,7 +18,5 @@
 mod backend;
 pub use backend::HdfsBuilder as Hdfs;
 
-mod appender;
-mod error;
-mod pager;
+mod lister;
 mod writer;
