@@ -23,12 +23,19 @@
 mod azblob;
 #[cfg(feature = "services-azblob")]
 pub use azblob::Azblob;
+#[cfg(feature = "services-azblob")]
+pub use azblob::AzblobConfig;
 
 #[cfg(feature = "services-azdls")]
 mod azdls;
 #[cfg(feature = "services-azdls")]
 pub use azdls::Azdls;
 
+#[cfg(feature = "services-cloudflare-kv")]
+mod cloudflare_kv;
+#[cfg(feature = "services-cloudflare-kv")]
+pub use self::cloudflare_kv::CloudflareKv;
+
 #[cfg(feature = "services-cos")]
 mod cos;
 #[cfg(feature = "services-cos")]
@@ -43,6 +50,8 @@ pub use self::dashmap::Dashmap;
 mod etcd;
 #[cfg(feature = "services-etcd")]
 pub use self::etcd::Etcd;
+#[cfg(feature = "services-etcd")]
+pub use self::etcd::EtcdConfig;
 
 #[cfg(feature = "services-fs")]
 mod fs;
@@ -53,6 +62,8 @@ pub use fs::Fs;
 mod ftp;
 #[cfg(feature = "services-ftp")]
 pub use ftp::Ftp;
+#[cfg(feature = "services-ftp")]
+pub use ftp::FtpConfig;
 
 #[cfg(feature = "services-gcs")]
 mod gcs;
@@ -64,6 +75,11 @@ mod ghac;
 #[cfg(feature = "services-ghac")]
 pub use ghac::Ghac;
 
+#[cfg(feature = "services-gridfs")]
+mod gridfs;
+#[cfg(feature = "services-gridfs")]
+pub use gridfs::Gridfs;
+
 #[cfg(feature = "services-hdfs")]
 mod hdfs;
 #[cfg(feature = "services-hdfs")]
@@ -73,6 +89,8 @@ pub use hdfs::Hdfs;
 mod http;
 #[cfg(feature = "services-http")]
 pub use self::http::Http;
+#[cfg(feature = "services-http")]
+pub use self::http::HttpConfig;
 
 #[cfg(feature = "services-ipfs")]
 mod ipfs;
@@ -84,6 +102,13 @@ mod ipmfs;
 #[cfg(feature = "services-ipmfs")]
 pub use ipmfs::Ipmfs;
 
+#[cfg(feature = "services-libsql")]
+mod libsql;
+#[cfg(feature = "services-libsql")]
+pub use libsql::Libsql;
+#[cfg(feature = "services-libsql")]
+pub use libsql::LibsqlConfig;
+
 #[cfg(feature = "services-memcached")]
 mod memcached;
 #[cfg(feature = "services-memcached")]
@@ -128,6 +153,8 @@ pub use self::persy::Persy;
 mod redis;
 #[cfg(feature = "services-redis")]
 pub use self::redis::Redis;
+#[cfg(feature = "services-redis")]
+pub use self::redis::RedisConfig;
 
 #[cfg(feature = "services-rocksdb")]
 mod rocksdb;
@@ -137,12 +164,16 @@ pub use self::rocksdb::Rocksdb;
 #[cfg(feature = "services-s3")]
 mod s3;
 #[cfg(feature = "services-s3")]
+pub use s3::S3Config;
+#[cfg(feature = "services-s3")]
 pub use s3::S3;
 
 #[cfg(feature = "services-sftp")]
 mod sftp;
 #[cfg(feature = "services-sftp")]
 pub use sftp::Sftp;
+#[cfg(feature = "services-sftp")]
+pub use sftp::SftpConfig;
 
 #[cfg(feature = "services-sled")]
 mod sled;
@@ -154,11 +185,6 @@ mod supabase;
 #[cfg(feature = "services-supabase")]
 pub use supabase::Supabase;
 
-#[cfg(feature = "services-wasabi")]
-mod wasabi;
-#[cfg(feature = "services-wasabi")]
-pub use wasabi::Wasabi;
-
 #[cfg(feature = "services-webdav")]
 mod webdav;
 #[cfg(feature = "services-webdav")]
@@ -198,6 +224,8 @@ pub use self::redb::Redb;
 mod tikv;
 #[cfg(feature = "services-tikv")]
 pub use self::tikv::Tikv;
+#[cfg(feature = "services-tikv")]
+pub use self::tikv::TikvConfig;
 
 #[cfg(feature = "services-foundationdb")]
 mod foundationdb;
@@ -208,6 +236,8 @@ pub use self::foundationdb::Foundationdb;
 mod postgresql;
 #[cfg(feature = "services-postgresql")]
 pub use self::postgresql::Postgresql;
+#[cfg(feature = "services-postgresql")]
+pub use self::postgresql::PostgresqlConfig;
 
 #[cfg(feature = "services-atomicserver")]
 mod atomicserver;
@@ -218,8 +248,51 @@ pub use self::atomicserver::Atomicserver;
 mod mysql;
 #[cfg(feature = "services-mysql")]
 pub use self::mysql::Mysql;
+#[cfg(feature = "services-mysql")]
+pub use self::mysql::MysqlConfig;
 
 #[cfg(feature = "services-sqlite")]
 mod sqlite;
 #[cfg(feature = "services-sqlite")]
-pub use self::sqlite::Sqlite;
+pub use sqlite::Sqlite;
+#[cfg(feature = "services-sqlite")]
+pub use sqlite::SqliteConfig;
+
+#[cfg(feature = "services-d1")]
+mod d1;
+#[cfg(feature = "services-d1")]
+pub use self::d1::D1;
+
+#[cfg(feature = "services-azfile")]
+mod azfile;
+#[cfg(feature = "services-azfile")]
+pub use self::azfile::Azfile;
+
+#[cfg(feature = "services-mongodb")]
+mod mongodb;
+#[cfg(feature = "services-mongodb")]
+pub use self::mongodb::Mongodb;
+
+#[cfg(feature = "services-dbfs")]
+mod dbfs;
+#[cfg(feature = "services-dbfs")]
+pub use self::dbfs::Dbfs;
+
+#[cfg(feature = "services-swift")]
+mod swift;
+#[cfg(feature = "services-swift")]
+pub use self::swift::Swift;
+
+#[cfg(feature = "services-alluxio")]
+mod alluxio;
+#[cfg(feature = "services-alluxio")]
+pub use alluxio::Alluxio;
+#[cfg(feature = "services-alluxio")]
+pub use alluxio::AlluxioConfig;
+
+#[cfg(feature = "services-b2")]
+mod b2;
+#[cfg(feature = "services-b2")]
+pub use b2::B2Config;
+#[cfg(feature = "services-b2")]
+pub use b2::B2;
