@@ -15,29 +15,58 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use std::cmp::min;
 use std::collections::HashMap;
 use std::fmt::Debug;
 use std::fmt::Formatter;
 use std::path::Path;
 use std::path::PathBuf;
+use std::pin::Pin;
 
 use async_trait::async_trait;
 use futures::StreamExt;
 use log::debug;
 use openssh::KnownHosts;
 use openssh::SessionBuilder;
+use openssh_sftp_client::file::TokioCompatFile;
 use openssh_sftp_client::Sftp;
 use openssh_sftp_client::SftpOptions;
+use serde::Deserialize;
 
 use super::error::is_not_found;
 use super::error::is_sftp_protocol_error;
-use super::pager::SftpPager;
-use super::utils::SftpReader;
+use super::lister::SftpLister;
 use super::writer::SftpWriter;
 use crate::raw::*;
 use crate::*;
 
+/// Config for Sftpservices support.
+#[derive(Default, Deserialize)]
+#[serde(default)]
+#[non_exhaustive]
+pub struct SftpConfig {
+    /// endpoint of this backend
+    pub endpoint: Option<String>,
+    /// root of this backend
+    pub root: Option<String>,
+    /// user of this backend
+    pub user: Option<String>,
+    /// key of this backend
+    pub key: Option<String>,
+    /// known_hosts_strategy of this backend
+    pub known_hosts_strategy: Option<String>,
+    /// enable_copy of this backend
+    pub enable_copy: bool,
+}
+
+impl Debug for SftpConfig {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("SftpConfig")
+            .field("endpoint", &self.endpoint)
+            .field("root", &self.root)
+            .finish_non_exhaustive()
+    }
+}
+
 /// SFTP services support. (only works on unix)
 ///
 /// If you are interested in working on windows, please refer to [this](https://github.com/apache/incubator-opendal/issues/2963) issue.
@@ -50,19 +79,13 @@ use crate::*;
 #[doc = include_str!("docs.md")]
 #[derive(Default)]
 pub struct SftpBuilder {
-    endpoint: Option<String>,
-    root: Option<String>,
-    user: Option<String>,
-    key: Option<String>,
-    known_hosts_strategy: Option<String>,
-    enable_copy: bool,
+    config: SftpConfig,
 }
 
 impl Debug for SftpBuilder {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        f.debug_struct("Builder")
-            .field("endpoint", &self.endpoint)
-            .field("root", &self.root)
+        f.debug_struct("SftpBuilder")
+            .field("config", &self.config)
             .finish()
     }
 }
@@ -70,7 +93,7 @@ impl Debug for SftpBuilder {
 impl SftpBuilder {
     /// set endpoint for sftp backend.
     pub fn endpoint(&mut self, endpoint: &str) -> &mut Self {
-        self.endpoint = if endpoint.is_empty() {
+        self.config.endpoint = if endpoint.is_empty() {
             None
         } else {
             Some(endpoint.to_string())
@@ -82,7 +105,7 @@ impl SftpBuilder {
     /// set root path for sftp backend.
     /// It uses the default directory set by the remote `sftp-server` as default.
     pub fn root(&mut self, root: &str) -> &mut Self {
-        self.root = if root.is_empty() {
+        self.config.root = if root.is_empty() {
             None
         } else {
             Some(root.to_string())
@@ -93,7 +116,7 @@ impl SftpBuilder {
 
     /// set user for sftp backend.
     pub fn user(&mut self, user: &str) -> &mut Self {
-        self.user = if user.is_empty() {
+        self.config.user = if user.is_empty() {
             None
         } else {
             Some(user.to_string())
@@ -104,7 +127,7 @@ impl SftpBuilder {
 
     /// set key path for sftp backend.
     pub fn key(&mut self, key: &str) -> &mut Self {
-        self.key = if key.is_empty() {
+        self.config.key = if key.is_empty() {
             None
         } else {
             Some(key.to_string())
@@ -119,7 +142,7 @@ impl SftpBuilder {
     /// - Accept
     /// - Add
     pub fn known_hosts_strategy(&mut self, strategy: &str) -> &mut Self {
-        self.known_hosts_strategy = if strategy.is_empty() {
+        self.config.known_hosts_strategy = if strategy.is_empty() {
             None
         } else {
             Some(strategy.to_string())
@@ -131,7 +154,7 @@ impl SftpBuilder {
     /// set enable_copy for sftp backend.
     /// It requires the server supports copy-file extension.
     pub fn enable_copy(&mut self, enable_copy: bool) -> &mut Self {
-        self.enable_copy = enable_copy;
+        self.config.enable_copy = enable_copy;
 
         self
     }
@@ -143,23 +166,24 @@ impl Builder for SftpBuilder {
 
     fn build(&mut self) -> Result<Self::Accessor> {
         debug!("sftp backend build started: {:?}", &self);
-        let endpoint = match self.endpoint.clone() {
+        let endpoint = match self.config.endpoint.clone() {
             Some(v) => v,
             None => return Err(Error::new(ErrorKind::ConfigInvalid, "endpoint is empty")),
         };
 
-        let user = match self.user.clone() {
+        let user = match self.config.user.clone() {
             Some(v) => v,
             None => return Err(Error::new(ErrorKind::ConfigInvalid, "user is empty")),
         };
 
         let root = self
+            .config
             .root
             .clone()
             .map(|r| normalize_root(r.as_str()))
             .unwrap_or_default();
 
-        let known_hosts_strategy = match &self.known_hosts_strategy {
+        let known_hosts_strategy = match &self.config.known_hosts_strategy {
             Some(v) => {
                 let v = v.to_lowercase();
                 if v == "strict" {
@@ -184,24 +208,18 @@ impl Builder for SftpBuilder {
             endpoint,
             root,
             user,
-            key: self.key.clone(),
+            key: self.config.key.clone(),
             known_hosts_strategy,
-            copyable: self.enable_copy,
+            copyable: self.config.enable_copy,
             client: tokio::sync::OnceCell::new(),
         })
     }
 
     fn from_map(map: HashMap<String, String>) -> Self {
-        let mut builder = SftpBuilder::default();
-
-        map.get("root").map(|v| builder.root(v));
-        map.get("endpoint").map(|v| builder.endpoint(v));
-        map.get("user").map(|v| builder.user(v));
-        map.get("key").map(|v| builder.key(v));
-        map.get("known_hosts_strategy")
-            .map(|v| builder.known_hosts_strategy(v));
-
-        builder
+        SftpBuilder {
+            config: SftpConfig::deserialize(ConfigDeserializer::new(map))
+                .expect("config deserialize must succeed"),
+        }
     }
 }
 
@@ -224,12 +242,12 @@ impl Debug for SftpBackend {
 
 #[async_trait]
 impl Accessor for SftpBackend {
-    type Reader = SftpReader;
+    type Reader = oio::TokioReader<Pin<Box<TokioCompatFile>>>;
     type BlockingReader = ();
     type Writer = SftpWriter;
     type BlockingWriter = ();
-    type Pager = Option<SftpPager>;
-    type BlockingPager = ();
+    type Lister = Option<SftpLister>;
+    type BlockingLister = ();
 
     fn info(&self) -> AccessorInfo {
         let mut am = AccessorInfo::default();
@@ -239,7 +257,6 @@ impl Accessor for SftpBackend {
                 stat: true,
 
                 read: true,
-                read_with_range: true,
                 read_can_seek: true,
 
                 write: true,
@@ -250,7 +267,7 @@ impl Accessor for SftpBackend {
 
                 list: true,
                 list_with_limit: true,
-                list_with_delimiter_slash: true,
+                list_without_recursive: true,
 
                 copy: self.copyable,
                 rename: true,
@@ -284,42 +301,24 @@ impl Accessor for SftpBackend {
         return Ok(RpCreateDir::default());
     }
 
-    async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
+    async fn read(&self, path: &str, _: OpRead) -> Result<(RpRead, Self::Reader)> {
         let client = self.connect().await?;
 
         let mut fs = client.fs();
         fs.set_cwd(&self.root);
         let path = fs.canonicalize(path).await?;
 
-        let mut file = client.open(path.as_path()).await?;
+        let f = client.open(path.as_path()).await?;
 
-        let total_length = file.metadata().await?.len().ok_or(Error::new(
-            ErrorKind::NotFound,
-            format!("file not found: {}", path.to_str().unwrap()).as_str(),
-        ))?;
+        // Sorry for the ugly code...
+        //
+        // - `f` is a openssh file.
+        // - `TokioCompatFile::new(f)` makes it implements tokio AsyncRead + AsyncSeek for openssh File.
+        // - `Box::pin(x)` to make sure this reader implements `Unpin`, since `TokioCompatFile` is not.
+        // - `oio::TokioReader::new(x)` makes it a `oio::TokioReader` which implements `oio::Read`.
+        let r = oio::TokioReader::new(Box::pin(TokioCompatFile::new(f)));
 
-        let br = args.range();
-        let (start, end) = match (br.offset(), br.size()) {
-            // Read a specific range.
-            (Some(offset), Some(size)) => (offset, min(offset + size, total_length)),
-            // Read from offset.
-            (Some(offset), None) => (offset, total_length),
-            // Read the last size bytes.
-            (None, Some(size)) => (
-                if total_length > size {
-                    total_length - size
-                } else {
-                    0
-                },
-                total_length,
-            ),
-            // Read the whole file.
-            (None, None) => (0, total_length),
-        };
-
-        let r = SftpReader::new(file, start, end).await?;
-
-        Ok((RpRead::new(end - start), r))
+        Ok((RpRead::new(), r))
     }
 
     async fn write(&self, path: &str, op: OpWrite) -> Result<(RpWrite, Self::Writer)> {
@@ -385,9 +384,9 @@ impl Accessor for SftpBackend {
         let mut fs = client.fs();
         fs.set_cwd(&self.root);
 
-        let meta = fs.metadata(path).await?;
+        let meta: Metadata = fs.metadata(path).await?.into();
 
-        Ok(RpStat::new(meta.into()))
+        Ok(RpStat::new(meta))
     }
 
     async fn delete(&self, path: &str, _: OpDelete) -> Result<RpDelete> {
@@ -446,7 +445,7 @@ impl Accessor for SftpBackend {
         Ok(RpDelete::default())
     }
 
-    async fn list(&self, path: &str, args: OpList) -> Result<(RpList, Self::Pager)> {
+    async fn list(&self, path: &str, _: OpList) -> Result<(RpList, Self::Lister)> {
         let client = self.connect().await?;
         let mut fs = client.fs();
         fs.set_cwd(&self.root);
@@ -467,7 +466,7 @@ impl Accessor for SftpBackend {
 
         Ok((
             RpList::default(),
-            Some(SftpPager::new(dir, path.to_owned(), args.limit())),
+            Some(SftpLister::new(dir, path.to_owned())),
         ))
     }
 }
