@@ -31,8 +31,8 @@ use http::StatusCode;
 use log::debug;
 
 use super::error::parse_error;
-use super::pager::Multistatus;
-use super::pager::WebdavPager;
+use super::lister::Multistatus;
+use super::lister::WebdavLister;
 use super::writer::WebdavWriter;
 use crate::raw::*;
 use crate::*;
@@ -223,8 +223,8 @@ impl Accessor for WebdavBackend {
     type BlockingReader = ();
     type Writer = oio::OneShotWriter<WebdavWriter>;
     type BlockingWriter = ();
-    type Pager = Option<WebdavPager>;
-    type BlockingPager = ();
+    type Lister = Option<oio::PageLister<WebdavLister>>;
+    type BlockingLister = ();
 
     fn info(&self) -> AccessorInfo {
         let mut ma = AccessorInfo::default();
@@ -248,7 +248,7 @@ impl Accessor for WebdavBackend {
                 rename: true,
 
                 list: true,
-                list_with_delimiter_slash: true,
+                list_without_recursive: true,
 
                 ..Default::default()
             });
@@ -268,9 +268,10 @@ impl Accessor for WebdavBackend {
         let status = resp.status();
         match status {
             StatusCode::OK | StatusCode::PARTIAL_CONTENT => {
-                let meta = parse_into_metadata(path, resp.headers())?;
-                Ok((RpRead::with_metadata(meta), resp.into_body()))
+                let size = parse_content_length(resp.headers())?;
+                Ok((RpRead::new().with_size(size), resp.into_body()))
             }
+            StatusCode::RANGE_NOT_SATISFIABLE => Ok((RpRead::new(), IncomingAsyncBody::empty())),
             _ => Err(parse_error(resp).await?),
         }
     }
@@ -325,11 +326,6 @@ impl Accessor for WebdavBackend {
     }
 
     async fn stat(&self, path: &str, _: OpStat) -> Result<RpStat> {
-        // Stat root always returns a DIR.
-        if path == "/" {
-            return Ok(RpStat::new(Metadata::new(EntryMode::DIR)));
-        }
-
         let mut header_map = HeaderMap::new();
         // not include children
         header_map.insert("Depth", "0".parse().unwrap());
@@ -340,30 +336,23 @@ impl Accessor for WebdavBackend {
         let status = resp.status();
 
         if !status.is_success() {
-            match status {
-                // HTTP Server like nginx could return FORBIDDEN if auto-index
-                // is not enabled, we should ignore them.
-                StatusCode::NOT_FOUND | StatusCode::FORBIDDEN if path.ends_with('/') => {
-                    Ok(RpStat::new(Metadata::new(EntryMode::DIR)))
-                }
-                _ => Err(parse_error(resp).await?),
-            }
-        } else {
-            let bs = resp.into_body().bytes().await?;
-            let result: Multistatus =
-                quick_xml::de::from_reader(bs.reader()).map_err(new_xml_deserialize_error)?;
-            let item = result
-                .response
-                .get(0)
-                .ok_or_else(|| {
-                    Error::new(
-                        ErrorKind::Unexpected,
-                        "Failed getting item stat: bad response",
-                    )
-                })?
-                .parse_into_metadata()?;
-            Ok(RpStat::new(item))
+            return Err(parse_error(resp).await?);
         }
+
+        let bs = resp.into_body().bytes().await?;
+        let result: Multistatus =
+            quick_xml::de::from_reader(bs.reader()).map_err(new_xml_deserialize_error)?;
+        let item = result
+            .response
+            .get(0)
+            .ok_or_else(|| {
+                Error::new(
+                    ErrorKind::Unexpected,
+                    "Failed getting item stat: bad response",
+                )
+            })?
+            .parse_into_metadata()?;
+        Ok(RpStat::new(item))
     }
 
     async fn delete(&self, path: &str, _: OpDelete) -> Result<RpDelete> {
@@ -377,11 +366,11 @@ impl Accessor for WebdavBackend {
         }
     }
 
-    async fn list(&self, path: &str, args: OpList) -> Result<(RpList, Self::Pager)> {
-        if args.delimiter() != "/" {
+    async fn list(&self, path: &str, args: OpList) -> Result<(RpList, Self::Lister)> {
+        if args.recursive() {
             return Err(Error::new(
                 ErrorKind::Unsupported,
-                "webdav only support delimiter `/`",
+                "webdav doesn't support list with recursive",
             ));
         }
 
@@ -397,10 +386,9 @@ impl Accessor for WebdavBackend {
                 let result: Multistatus =
                     quick_xml::de::from_reader(bs.reader()).map_err(new_xml_deserialize_error)?;
 
-                Ok((
-                    RpList::default(),
-                    Some(WebdavPager::new(&self.base_dir, &self.root, path, result)),
-                ))
+                let l = WebdavLister::new(&self.base_dir, &self.root, path, result);
+
+                Ok((RpList::default(), Some(oio::PageLister::new(l))))
             }
             StatusCode::NOT_FOUND if path.ends_with('/') => Ok((RpList::default(), None)),
             _ => Err(parse_error(resp).await?),
