@@ -20,14 +20,13 @@ use std::time::Duration;
 use bytes::Buf;
 use bytes::Bytes;
 use futures::stream;
-use futures::AsyncReadExt;
 use futures::Stream;
 use futures::StreamExt;
 use futures::TryStreamExt;
-use tokio::io::ReadBuf;
 
 use super::BlockingOperator;
 use crate::operator_futures::*;
+use crate::raw::oio::ReadExt;
 use crate::raw::oio::WriteExt;
 use crate::raw::*;
 use crate::*;
@@ -159,16 +158,44 @@ impl Operator {
         }
     }
 
-    /// Get current path's metadata.
+    /// Get given path's metadata.
     ///
     /// # Notes
     ///
     /// For fetch metadata of entries returned by [`Lister`], it's better to use [`list_with`] and
     /// [`lister_with`] with `metakey` query like `Metakey::ContentLength | Metakey::LastModified`
-    /// so that we can avoid extra requests.
+    /// so that we can avoid extra stat requests.
+    ///
+    /// # Behavior
+    ///
+    /// ## Services that support `create_dir`
+    ///
+    /// `test` and `test/` may vary in some services such as S3. However, on a local file system,
+    /// they're identical. Therefore, the behavior of `stat("test")` and `stat("test/")` might differ
+    /// in certain edge cases. Always use `stat("test/")` when you need to access a directory if possible.
+    ///
+    /// Here are the behavior list:
+    ///
+    /// | Case                   | Path            | Result                                     |
+    /// |------------------------|-----------------|--------------------------------------------|
+    /// | stat existing dir      | `abc/`          | Metadata with dir mode                     |
+    /// | stat existing file     | `abc/def_file`  | Metadata with file mode                    |
+    /// | stat dir without `/`   | `abc/def_dir`   | Error `NotFound` or metadata with dir mode |
+    /// | stat file with `/`     | `abc/def_file/` | Error `NotFound`                           |
+    /// | stat not existing path | `xyz`           | Error `NotFound`                           |
+    ///
+    /// Refer to [RFC: List Prefix][crate::docs::rfcs::rfc_3243_list_prefix] for more details.
+    ///
+    /// ## Services that not support `create_dir`
+    ///
+    /// For services that not support `create_dir`, `stat("test/")` will return `NotFound` even
+    /// when `test/abc` exists since the service won't have the concept of dir. There is nothing
+    /// we can do about this.
     ///
     /// # Examples
     ///
+    /// ## Check if file exists
+    ///
     /// ```
     /// # use anyhow::Result;
     /// # use futures::io;
@@ -189,21 +216,51 @@ impl Operator {
         self.stat_with(path).await
     }
 
-    /// Get current path's metadata **without cache** directly with extra options.
+    /// Get given path's metadata with extra options.
     ///
     /// # Notes
     ///
-    /// Use `stat` if you:
+    /// For fetch metadata of entries returned by [`Lister`], it's better to use [`list_with`] and
+    /// [`lister_with`] with `metakey` query like `Metakey::ContentLength | Metakey::LastModified`
+    /// so that we can avoid extra requests.
     ///
-    /// - Want to detect the outside changes of path.
-    /// - Don't want to read from cached metadata.
+    /// # Behavior
     ///
-    /// You may want to use `metadata` if you are working with entries
-    /// returned by [`Lister`]. It's highly possible that metadata
-    /// you want has already been cached.
+    /// ## Services that support `create_dir`
+    ///
+    /// `test` and `test/` may vary in some services such as S3. However, on a local file system,
+    /// they're identical. Therefore, the behavior of `stat("test")` and `stat("test/")` might differ
+    /// in certain edge cases. Always use `stat("test/")` when you need to access a directory if possible.
+    ///
+    /// Here are the behavior list:
+    ///
+    /// | Case                   | Path            | Result                                     |
+    /// |------------------------|-----------------|--------------------------------------------|
+    /// | stat existing dir      | `abc/`          | Metadata with dir mode                     |
+    /// | stat existing file     | `abc/def_file`  | Metadata with file mode                    |
+    /// | stat dir without `/`   | `abc/def_dir`   | Error `NotFound` or metadata with dir mode |
+    /// | stat file with `/`     | `abc/def_file/` | Error `NotFound`                           |
+    /// | stat not existing path | `xyz`           | Error `NotFound`                           |
+    ///
+    /// Refer to [RFC: List Prefix][crate::docs::rfcs::rfc_3243_list_prefix] for more details.
+    ///
+    /// ## Services that not support `create_dir`
+    ///
+    /// For services that not support `create_dir`, `stat("test/")` will return `NotFound` even
+    /// when `test/abc` exists since the service won't have the concept of dir. There is nothing
+    /// we can do about this.
     ///
     /// # Examples
     ///
+    /// ## Get metadata while `ETag` matches
+    ///
+    /// `stat_with` will
+    ///
+    /// - return `Ok(metadata)` if `ETag` matches
+    /// - return `Err(error)` and `error.kind() == ErrorKind::ConditionNotMatch` if file exists but
+    ///   `ETag` mismatch
+    /// - return `Err(err)` if other errors occur, for example, `NotFound`.
+    ///
     /// ```
     /// # use anyhow::Result;
     /// # use futures::io;
@@ -213,6 +270,9 @@ impl Operator {
     /// # #[tokio::main]
     /// # async fn test(op: Operator) -> Result<()> {
     /// if let Err(e) = op.stat_with("test").if_match("<etag>").await {
+    ///     if e.kind() == ErrorKind::ConditionNotMatch {
+    ///         println!("file exists, but etag mismatch")
+    ///     }
     ///     if e.kind() == ErrorKind::NotFound {
     ///         println!("file not exist")
     ///     }
@@ -368,32 +428,25 @@ impl Operator {
                         .with_context("path", &path));
                     }
 
-                    let br = args.range();
-                    let (rp, mut s) = inner.read(&path, args).await?;
+                    let range = args.range();
+                    let size_hint = match range.size() {
+                        Some(v) => v,
+                        None => {
+                            let mut size = inner
+                                .stat(&path, OpStat::default())
+                                .await?
+                                .into_metadata()
+                                .content_length();
+                            size -= range.offset().unwrap_or(0);
+                            size
+                        }
+                    };
 
-                    let length = rp.into_metadata().content_length() as usize;
-                    let mut buffer = Vec::with_capacity(length);
+                    let (_, mut s) = inner.read(&path, args).await?;
+                    let mut buf = Vec::with_capacity(size_hint as usize);
+                    s.read_to_end(&mut buf).await?;
 
-                    let dst = buffer.spare_capacity_mut();
-                    let mut buf = ReadBuf::uninit(dst);
-
-                    // Safety: the input buffer is created with_capacity(length).
-                    unsafe { buf.assume_init(length) };
-
-                    // TODO: use native read api
-                    s.read_exact(buf.initialized_mut()).await.map_err(|err| {
-                        Error::new(ErrorKind::Unexpected, "read from storage")
-                            .with_operation("read")
-                            .with_context("service", inner.info().scheme().into_static())
-                            .with_context("path", &path)
-                            .with_context("range", br.to_string())
-                            .set_source(err)
-                    })?;
-
-                    // Safety: read_exact makes sure this buffer has been filled.
-                    unsafe { buffer.set_len(length) }
-
-                    Ok(buffer)
+                    Ok(buf)
                 };
 
                 Box::pin(fut)
@@ -858,6 +911,8 @@ impl Operator {
     /// # }
     /// ```
     pub async fn remove_via(&self, input: impl Stream<Item = String> + Unpin) -> Result<()> {
+        let input = input.map(|v| normalize_path(&v));
+
         if self.info().full_capability().batch {
             let mut input = input
                 .map(|v| (v, OpDelete::default().into()))
@@ -924,7 +979,7 @@ impl Operator {
             return self.delete(path).await;
         }
 
-        let obs = self.lister_with(path).delimiter("").await?;
+        let obs = self.lister_with(path).recursive(true).await?;
 
         if self.info().full_capability().batch {
             let mut obs = obs.try_chunks(self.limit());
@@ -965,7 +1020,7 @@ impl Operator {
     /// ## Listing recursively
     ///
     /// This function only read the children of the given directory. To read
-    /// all entries recursively, use `Operator::list_with("path").delimiter("")`
+    /// all entries recursively, use `Operator::list_with("path").recursive(true)`
     /// instead.
     ///
     /// ## Streaming
@@ -1007,7 +1062,7 @@ impl Operator {
     /// # }
     /// ```
     pub async fn list(&self, path: &str) -> Result<Vec<Entry>> {
-        self.lister_with(path).await?.try_collect().await
+        self.list_with(path).await
     }
 
     /// List entries within a given directory with options.
@@ -1041,7 +1096,7 @@ impl Operator {
     /// use opendal::Operator;
     /// # #[tokio::main]
     /// # async fn test(op: Operator) -> Result<()> {
-    /// let mut entries = op.list_with("prefix/").delimiter("").await?;
+    /// let mut entries = op.list_with("prefix/").recursive(true).await?;
     /// for entry in entries {
     ///     match entry.metadata().mode() {
     ///         EntryMode::FILE => {
@@ -1129,7 +1184,7 @@ impl Operator {
     /// ## Listing recursively
     ///
     /// This function only read the children of the given directory. To read
-    /// all entries recursively, use [`Operator::lister_with`] and `delimiter("")`
+    /// all entries recursively, use [`Operator::lister_with`] and `recursive(true)`
     /// instead.
     ///
     /// ## Metadata
@@ -1217,7 +1272,7 @@ impl Operator {
     /// use opendal::Operator;
     /// # #[tokio::main]
     /// # async fn test(op: Operator) -> Result<()> {
-    /// let mut ds = op.lister_with("path/to/dir/").delimiter("").await?;
+    /// let mut ds = op.lister_with("path/to/dir/").recursive(true).await?;
     /// while let Some(mut entry) = ds.try_next().await? {
     ///     match entry.metadata().mode() {
     ///         EntryMode::FILE => {
