@@ -64,7 +64,7 @@ pub async fn test_list_dir(op: Operator) -> Result<()> {
     let parent = uuid::Uuid::new_v4().to_string();
     let path = format!("{parent}/{}", uuid::Uuid::new_v4());
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content).await.expect("write must succeed");
 
@@ -91,7 +91,7 @@ pub async fn test_list_dir_with_metakey(op: Operator) -> Result<()> {
     let parent = uuid::Uuid::new_v4().to_string();
     let path = format!("{parent}/{}", uuid::Uuid::new_v4());
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content).await.expect("write must succeed");
 
@@ -141,7 +141,7 @@ pub async fn test_list_dir_with_metakey_complete(op: Operator) -> Result<()> {
     let parent = uuid::Uuid::new_v4().to_string();
     let path = format!("{parent}/{}", uuid::Uuid::new_v4());
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content).await.expect("write must succeed");
 
@@ -366,7 +366,7 @@ pub async fn test_list_with_start_after(op: Operator) -> Result<()> {
 }
 
 pub async fn test_scan_root(op: Operator) -> Result<()> {
-    let w = op.lister_with("").delimiter("").await?;
+    let w = op.lister_with("").recursive(true).await?;
     let actual = w
         .try_collect::<Vec<_>>()
         .await?
@@ -396,7 +396,7 @@ pub async fn test_scan(op: Operator) -> Result<()> {
 
     let w = op
         .lister_with(&format!("{parent}/x/"))
-        .delimiter("")
+        .recursive(true)
         .await?;
     let actual = w
         .try_collect::<Vec<_>>()
