@@ -56,6 +56,7 @@ pub fn behavior_write_tests(op: &Operator) -> Vec<Trial> {
         test_write_with_content_disposition,
         test_stat_file,
         test_stat_dir,
+        test_stat_nested_parent_dir,
         test_stat_with_special_chars,
         test_stat_not_cleaned_path,
         test_stat_not_exist,
@@ -97,6 +98,10 @@ pub fn behavior_write_tests(op: &Operator) -> Vec<Trial> {
 
 /// Create dir with dir path should succeed.
 pub async fn test_create_dir(op: Operator) -> Result<()> {
+    if !op.info().full_capability().create_dir {
+        return Ok(());
+    }
+
     let path = format!("{}/", uuid::Uuid::new_v4());
 
     op.create_dir(&path).await?;
@@ -110,6 +115,10 @@ pub async fn test_create_dir(op: Operator) -> Result<()> {
 
 /// Create dir on existing dir should succeed.
 pub async fn test_create_dir_existing(op: Operator) -> Result<()> {
+    if !op.info().full_capability().create_dir {
+        return Ok(());
+    }
+
     let path = format!("{}/", uuid::Uuid::new_v4());
 
     op.create_dir(&path).await?;
@@ -126,7 +135,7 @@ pub async fn test_create_dir_existing(op: Operator) -> Result<()> {
 /// Write a single file and test with stat.
 pub async fn test_write_only(op: Operator) -> Result<()> {
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content).await?;
 
@@ -157,7 +166,7 @@ pub async fn test_write_with_empty_content(op: Operator) -> Result<()> {
 /// Write file with dir path should return an error
 pub async fn test_write_with_dir_path(op: Operator) -> Result<()> {
     let path = format!("{}/", uuid::Uuid::new_v4());
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     let result = op.write(&path, content).await;
     assert!(result.is_err());
@@ -180,7 +189,7 @@ pub async fn test_write_with_special_chars(op: Operator) -> Result<()> {
     }
 
     let path = format!("{} !@#$%^&()_+-=;',.txt", uuid::Uuid::new_v4());
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content).await?;
 
@@ -198,7 +207,7 @@ pub async fn test_write_with_cache_control(op: Operator) -> Result<()> {
     }
 
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     let target_cache_control = "no-cache, no-store, max-age=300";
     op.write_with(&path, content)
@@ -224,7 +233,7 @@ pub async fn test_write_with_content_type(op: Operator) -> Result<()> {
     }
 
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     let target_content_type = "application/json";
     op.write_with(&path, content)
@@ -251,7 +260,7 @@ pub async fn test_write_with_content_disposition(op: Operator) -> Result<()> {
     }
 
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     let target_content_disposition = "attachment; filename=\"filename.jpg\"";
     op.write_with(&path, content)
@@ -274,7 +283,7 @@ pub async fn test_write_with_content_disposition(op: Operator) -> Result<()> {
 /// Stat existing file should return metadata
 pub async fn test_stat_file(op: Operator) -> Result<()> {
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content).await.expect("write must succeed");
 
@@ -282,12 +291,23 @@ pub async fn test_stat_file(op: Operator) -> Result<()> {
     assert_eq!(meta.mode(), EntryMode::FILE);
     assert_eq!(meta.content_length(), size as u64);
 
+    // Stat a file with trailing slash should return `NotFound`.
+    if op.info().full_capability().create_dir {
+        let result = op.stat(&format!("{path}/")).await;
+        assert!(result.is_err());
+        assert_eq!(result.unwrap_err().kind(), ErrorKind::NotFound);
+    }
+
     op.delete(&path).await.expect("delete must succeed");
     Ok(())
 }
 
 /// Stat existing file should return metadata
 pub async fn test_stat_dir(op: Operator) -> Result<()> {
+    if !op.info().full_capability().create_dir {
+        return Ok(());
+    }
+
     let path = format!("{}/", uuid::Uuid::new_v4());
 
     op.create_dir(&path).await.expect("write must succeed");
@@ -295,10 +315,40 @@ pub async fn test_stat_dir(op: Operator) -> Result<()> {
     let meta = op.stat(&path).await?;
     assert_eq!(meta.mode(), EntryMode::DIR);
 
+    // Stat a dir without trailing slash could have two behavior.
+    let result = op.stat(path.trim_end_matches('/')).await;
+    match result {
+        Ok(meta) => assert_eq!(meta.mode(), EntryMode::DIR),
+        Err(err) => assert_eq!(err.kind(), ErrorKind::NotFound),
+    }
+
     op.delete(&path).await.expect("delete must succeed");
     Ok(())
 }
 
+/// Stat the parent dir of existing dir should return metadata
+pub async fn test_stat_nested_parent_dir(op: Operator) -> Result<()> {
+    if !op.info().full_capability().create_dir {
+        return Ok(());
+    }
+
+    let parent = format!("{}", uuid::Uuid::new_v4());
+    let file = format!("{}", uuid::Uuid::new_v4());
+    let (content, _) = gen_bytes(op.info().full_capability());
+
+    op.write(&format!("{parent}/{file}"), content.clone())
+        .await
+        .expect("write must succeed");
+
+    let meta = op.stat(&format!("{parent}/")).await?;
+    assert_eq!(meta.mode(), EntryMode::DIR);
+
+    op.delete(&format!("{parent}/{file}"))
+        .await
+        .expect("delete must succeed");
+    Ok(())
+}
+
 /// Stat existing file with special chars should return metadata
 pub async fn test_stat_with_special_chars(op: Operator) -> Result<()> {
     // Ignore test for supabase until https://github.com/apache/incubator-opendal/issues/2194 addressed.
@@ -313,7 +363,7 @@ pub async fn test_stat_with_special_chars(op: Operator) -> Result<()> {
     }
 
     let path = format!("{} !@#$%^&()_+-=;',.txt", uuid::Uuid::new_v4());
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content).await.expect("write must succeed");
 
@@ -329,7 +379,7 @@ pub async fn test_stat_with_special_chars(op: Operator) -> Result<()> {
 pub async fn test_stat_not_cleaned_path(op: Operator) -> Result<()> {
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content).await.expect("write must succeed");
 
@@ -345,10 +395,18 @@ pub async fn test_stat_not_cleaned_path(op: Operator) -> Result<()> {
 pub async fn test_stat_not_exist(op: Operator) -> Result<()> {
     let path = uuid::Uuid::new_v4().to_string();
 
+    // Stat not exist file should returns NotFound.
     let meta = op.stat(&path).await;
     assert!(meta.is_err());
     assert_eq!(meta.unwrap_err().kind(), ErrorKind::NotFound);
 
+    // Stat not exist dir should also returns NotFound.
+    if op.info().full_capability().create_dir {
+        let meta = op.stat(&format!("{path}/")).await;
+        assert!(meta.is_err());
+        assert_eq!(meta.unwrap_err().kind(), ErrorKind::NotFound);
+    }
+
     Ok(())
 }
 
@@ -360,7 +418,7 @@ pub async fn test_stat_with_if_match(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -392,7 +450,7 @@ pub async fn test_stat_with_if_none_match(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -435,7 +493,7 @@ pub async fn test_stat_root(op: Operator) -> Result<()> {
 pub async fn test_read_full(op: Operator) -> Result<()> {
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -461,7 +519,7 @@ pub async fn test_read_range(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
     let (offset, length) = gen_offset_length(size);
 
     op.write(&path, content.clone())
@@ -491,7 +549,7 @@ pub async fn test_read_large_range(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
     let (offset, _) = gen_offset_length(size);
 
     op.write(&path, content.clone())
@@ -522,7 +580,7 @@ pub async fn test_reader_range(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
     let (offset, length) = gen_offset_length(size);
 
     op.write(&path, content.clone())
@@ -555,7 +613,7 @@ pub async fn test_reader_from(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
     let (offset, _) = gen_offset_length(size);
 
     op.write(&path, content.clone())
@@ -586,7 +644,7 @@ pub async fn test_reader_tail(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
     let (_, length) = gen_offset_length(size);
 
     op.write(&path, content.clone())
@@ -636,7 +694,7 @@ pub async fn test_read_with_if_match(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -667,7 +725,7 @@ pub async fn test_read_with_if_none_match(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -700,7 +758,7 @@ pub async fn test_fuzz_reader_with_range(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -741,7 +799,7 @@ pub async fn test_fuzz_offset_reader(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -782,7 +840,7 @@ pub async fn test_fuzz_part_reader(op: Operator) -> Result<()> {
 
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
     let (offset, length) = gen_offset_length(size);
 
     op.write(&path, content.clone())
@@ -820,6 +878,10 @@ pub async fn test_fuzz_part_reader(op: Operator) -> Result<()> {
 
 /// Read with dir path should return an error.
 pub async fn test_read_with_dir_path(op: Operator) -> Result<()> {
+    if !op.info().full_capability().create_dir {
+        return Ok(());
+    }
+
     let path = format!("{}/", uuid::Uuid::new_v4());
 
     op.create_dir(&path).await.expect("write must succeed");
@@ -847,7 +909,7 @@ pub async fn test_read_with_special_chars(op: Operator) -> Result<()> {
 
     let path = format!("{} !@#$%^&()_+-=;',.txt", uuid::Uuid::new_v4());
     debug!("Generate a random file: {}", &path);
-    let (content, size) = gen_bytes();
+    let (content, size) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -874,7 +936,7 @@ pub async fn test_read_with_override_cache_control(op: Operator) -> Result<()> {
     }
 
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -924,7 +986,7 @@ pub async fn test_read_with_override_content_disposition(op: Operator) -> Result
     }
 
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -974,7 +1036,7 @@ pub async fn test_read_with_override_content_type(op: Operator) -> Result<()> {
     }
 
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
@@ -1018,7 +1080,7 @@ pub async fn test_read_with_override_content_type(op: Operator) -> Result<()> {
 /// Delete existing file should succeed.
 pub async fn test_writer_abort(op: Operator) -> Result<()> {
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     let mut writer = match op.writer(&path).await {
         Ok(writer) => writer,
@@ -1046,7 +1108,7 @@ pub async fn test_writer_abort(op: Operator) -> Result<()> {
 /// Delete existing file should succeed.
 pub async fn test_delete_file(op: Operator) -> Result<()> {
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content).await.expect("write must succeed");
 
@@ -1060,6 +1122,10 @@ pub async fn test_delete_file(op: Operator) -> Result<()> {
 
 /// Delete empty dir should succeed.
 pub async fn test_delete_empty_dir(op: Operator) -> Result<()> {
+    if !op.info().full_capability().create_dir {
+        return Ok(());
+    }
+
     let path = format!("{}/", uuid::Uuid::new_v4());
 
     op.create_dir(&path).await.expect("create must succeed");
@@ -1084,7 +1150,7 @@ pub async fn test_delete_with_special_chars(op: Operator) -> Result<()> {
 
     let path = format!("{} !@#$%^&()_+-=;',.txt", uuid::Uuid::new_v4());
     debug!("Generate a random file: {}", &path);
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content).await.expect("write must succeed");
 
@@ -1108,9 +1174,20 @@ pub async fn test_delete_not_existing(op: Operator) -> Result<()> {
 /// Remove one file
 pub async fn test_remove_one_file(op: Operator) -> Result<()> {
     let path = uuid::Uuid::new_v4().to_string();
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
-    op.write(&path, content).await.expect("write must succeed");
+    op.write(&path, content.clone())
+        .await
+        .expect("write must succeed");
+
+    op.remove(vec![path.clone()]).await?;
+
+    // Stat it again to check.
+    assert!(!op.is_exist(&path).await?);
+
+    op.write(&format!("/{path}"), content)
+        .await
+        .expect("write must succeed");
 
     op.remove(vec![path.clone()]).await?;
 
@@ -1122,6 +1199,10 @@ pub async fn test_remove_one_file(op: Operator) -> Result<()> {
 
 /// Delete via stream.
 pub async fn test_delete_stream(op: Operator) -> Result<()> {
+    if !op.info().full_capability().create_dir {
+        return Ok(());
+    }
+
     let dir = uuid::Uuid::new_v4().to_string();
     op.create_dir(&format!("{dir}/"))
         .await
@@ -1325,7 +1406,7 @@ pub async fn test_fuzz_unsized_writer(op: Operator) -> Result<()> {
 pub async fn test_invalid_reader_seek(op: Operator) -> Result<()> {
     let path = uuid::Uuid::new_v4().to_string();
     debug!("Generate a random file: {}", &path);
-    let (content, _) = gen_bytes();
+    let (content, _) = gen_bytes(op.info().full_capability());
 
     op.write(&path, content.clone())
         .await
